
#if (!defined(final_trace_number))
/* *********************
 * Change this number depending on which rendering
 * pass you want to do. Setting to zero will use
 * the development settings above. To produce a
 * final render, set the number sequentially to
 * values 1-4. Setting 1 creates a simple photon
 * map. Setting 2 generates radiosity data. Setting
 * 3 generates a more sophisticated photon map.
 * Setting 4 puts it all together and creates the
 * final image. Settings 1-3 shoud be rendered at
 * a low resolution, i.e. 160x120, no AA. Settings
 * 1 and 3 can be canceled after the pretrace is done,
 * if desired. Setting four should be rendered to
 * completion, and should be done at the desired
 * final resolution and quality.
 *
 * 0 - Custom render
 * 1 - Photons of mirror only
 * 2 - Radiosity calculations
 * 3 - Photons of all foreground blocks
 * 4 - Final trace (Photons and radiosity)
 * 5 - Final trace (Photons, No radiosity)
 * 6 - Final trace (Mirror photons only)
 * 7 - Final trace (No photons or radiosity)
 * 8 - Preview trace (No photons or radiosity)
 */

#declare final_trace_number = 8;











#end

#if (final_trace_number = 5)
    #default {
        finish { ambient 0.2 }
    }
#else
    #if (final_trace_number = 7 | final_trace_number = 8)
        #default {
            finish { ambient 0.42 }
        }
    #else
        #default {
            finish { ambient 0 }
        }
    #end
#end

#include "colors.inc" 
#include "math.inc"
#include "blocks.inc"
#include "glass.inc"
#include "wood_floor.inc"


/* ***********************
 * Development settings. Will be
 * used only when final_trace_number
 * is set to zero. Set close_camera
 * to 1 and low_light to 1 in order
 * to examine a single block in detail.
 */

#declare quick_radiosity=0;

#declare do_focal_blur=0;
#declare do_radiosity=0;
#declare do_area_light=0;
#declare do_open_window=1;
#declare do_photons=0;
#declare do_reflected_blocks=0;
#declare do_spotlights=0;
#declare do_extra_blocks=0;
#declare do_pyramid_blocks=1;
#declare do_mirror=0;
#declare do_close_camera=0;
#declare do_low_light=0;
#declare do_main_light=1;
#declare do_board_floor=0;
#declare save_photons=0;
#declare disable_photon_refraction=0;
#declare save_rad=0;
#declare simple_blocks=0;
#declare simple_textures=1;
#declare no_glass=1;
#declare use_photons=1;
#declare do_extra_lights=0;
#declare do_sun=1;
#declare point_sun=1;
#declare sun_power=3;
#declare sky_brightness=7;
#if (do_radiosity)
    #declare light_level=2.00;
#else
    #declare light_level=1.1;
#end


/* *******************************
 * The only photons I want to take into account
 * for radiosity calculations are those that
 * reflect off the back mirror.
 */

#if (final_trace_number = 1)
    #declare do_radiosity=0;
    #declare save_rad=0;
    #declare do_board_floor=1;
    #declare do_floor_photons=0;
    #declare do_photons=1;
    #declare disable_photon_refraction=0;
    #declare save_photons=1;
    #declare do_focal_blur=0;
    #declare do_area_light=0;
    #declare do_open_window=0;
    #declare do_sun=1;
    #declare point_sun=1;
    #declare sun_power=2.5;
    #declare do_mirror=1;
    #declare do_reflected_blocks=0;
    #declare do_pyramid_blocks=1;
    #declare do_extra_blocks=1;
    #declare do_close_camera=0;
    #declare do_low_light=0;
    #declare do_main_light=1;
    #declare light_level=2.2;
    #declare sky_brightness=15;
    #declare simple_blocks=1;
    #declare simple_textures=1;
    #declare no_glass=1;
    #declare use_photons=0;
    #declare do_extra_lights=0;
#end

/* ****************************
 * Photon refractions are disabled in all light
 * sources for this step because otherwise the
 * insides of the blocks are too dark! I want to
 * use the "simple blocks" in this step, but
 * the refractions seems to come out all wrong
 * when I do so. I don't have time to fiddle
 * with this right now.
 */

#if (final_trace_number = 2)
    #declare do_radiosity=1;
    #declare save_rad=1;
    #declare do_board_floor=1;
    #declare do_floor_photons=0;
    #declare do_photons=1;
    #declare disable_photon_refraction=1;
    #declare save_photons=0;
    #declare do_focal_blur=0;
    #declare do_area_light=0;
    #declare do_open_window=0;
    #declare do_sun=1;
    #declare point_sun=1;
    #declare sun_power=2.5;
    #declare do_mirror=1;
    #declare do_reflected_blocks=1;
    #declare do_pyramid_blocks=1;
    #declare do_extra_blocks=1;
    #declare do_close_camera=0;
    #declare do_low_light=0;
    #declare do_main_light=1;
    #declare light_level=1.05;
    #declare sky_brightness=15;
    #declare simple_blocks=0;
    #declare simple_textures=1;
    #declare no_glass=1;
    #declare use_photons=0;
    #declare do_extra_lights=0;
#end

/* ***************************
 * Now recalculate a more detailed photon map.
 * We are still ignoring caustics created due
 * to blocks reflected in the mirror. To do
 * otherwise creates rediculously large photon
 * maps, in excess of 1 GB.
 * NOTE: Increase the light_level for this step
 * to get more pronounced caustics.
 */

#if (final_trace_number = 3)
    #declare do_radiosity=0;
    #declare save_rad=0;
    #declare do_board_floor=1;
    #declare do_floor_photons=0;
    #declare do_photons=1;
    #declare save_photons=1;
    #declare disable_photon_refraction=0;
    #declare do_focal_blur=0;
    #declare do_area_light=0;
    #declare do_open_window=0;
    #declare do_sun=1;
    #declare point_sun=1;
    #declare sun_power=4;
    #declare do_mirror=1;
    #declare do_reflected_blocks=1;
    #declare do_pyramid_blocks=1;
    #declare do_extra_blocks=1;
    #declare do_close_camera=0;
    #declare do_low_light=0;
    #declare do_main_light=1;
    #declare light_level=2.2;
    #declare sky_brightness=15;
    #declare simple_blocks=0;
    #declare simple_textures=0;
    #declare no_glass=0;
    #declare use_photons=1;
    #declare do_extra_lights=0;
#end

/* ************************
 * This is the final step. Put the radiosity data
 * calculated in trace number 2 together with the
 * photon map calculated in trace number 3, and
 * render the resulting image with full quality
 * settings.
 */

#if (final_trace_number = 4)
    #declare do_radiosity=1;
    #declare save_rad=0;
    #declare do_board_floor=1;
    #declare do_floor_photons=0;
    #declare do_photons=1;
    #declare save_photons=0;
    #declare disable_photon_refraction=0;
    #declare do_focal_blur=0;
    #declare do_area_light=0;
    #declare do_open_window=0;
    #declare do_sun=1;
    #declare point_sun=1;
    #declare sun_power=2.5;
    #declare do_mirror=1;
    #declare do_reflected_blocks=1;
    #declare do_pyramid_blocks=1;
    #declare do_extra_blocks=1;
    #declare do_close_camera=0;
    #declare do_low_light=0;
    #declare do_main_light=1;
    #declare light_level=1.05;
    #declare sky_brightness=15;
    #declare simple_blocks=0;
    #declare simple_textures=0;
    #declare no_glass=0;
    #declare use_photons=1;
    #declare do_extra_lights=0;
#end

/* ************************
 * Trace number 5 renders the scene without
 * radiosity. In order to do trace 5, it must
 * be preceeded by trace 3. Traces 1, 2, and 4
 * are not needed for this one.
 */

#if (final_trace_number = 5)
    #declare do_focal_blur=0;
    #declare do_radiosity=0;
    #declare do_area_light=0;
    #declare do_open_window=0;
    #declare do_photons=1;
    #declare do_reflected_blocks=1;
    #declare do_spotlights=0;
    #declare do_extra_blocks=1;
    #declare do_pyramid_blocks=1;
    #declare do_mirror=1;
    #declare do_close_camera=0;
    #declare do_low_light=0;
    #declare do_main_light=1;
    #declare do_board_floor=1;
    #declare do_floor_photons=0;
    #declare save_photons=0;
    #declare disable_photon_refraction=0;
    #declare save_rad=0;
    #declare simple_blocks=0;
    #declare simple_textures=0;
    #declare no_glass=0;
    #declare use_photons=1;
    #declare do_extra_lights=0;
    #declare do_sun=1;
    #declare point_sun=1;
    #declare sun_power=3;
    #declare sky_brightness=7;
    #declare light_level=1.2;
#end

/* ************************
 * Trace number 6 renders the scene without
 * photons. In order to do trace 6, it must
 * be preceeded by traces 1 & 2. Traces 3 and 4
 * are not needed for this one.
 */

#if (final_trace_number = 6)
    #declare do_focal_blur=0;
    #declare do_radiosity=1;
    #declare do_area_light=0;
    #declare do_open_window=0;
    #declare do_photons=1;
    #declare do_reflected_blocks=1;
    #declare do_spotlights=0;
    #declare do_extra_blocks=1;
    #declare do_pyramid_blocks=1;
    #declare do_mirror=1;
    #declare do_close_camera=0;
    #declare do_low_light=0;
    #declare do_main_light=1;
    #declare do_board_floor=1;
    #declare do_floor_photons=0;
    #declare save_photons=0;
    #declare disable_photon_refraction=0;
    #declare save_rad=0;
    #declare simple_blocks=0;
    #declare simple_textures=0;
    #declare no_glass=0;
    #declare use_photons=0;
    #declare do_extra_lights=0;
    #declare do_sun=1;
    #declare point_sun=1;
    #declare sun_power=3;
    #declare sky_brightness=7;
    #declare light_level=1.05;
#end

/* ************************
 * Trace number 7 renders the scene without
 * radiosity or photons. No pretraces are
 * necessary for this setting.
 */

#if (final_trace_number = 7)
    #declare do_focal_blur=0;
    #declare do_radiosity=0;
    #declare do_area_light=0;
    #declare do_open_window=0;
    #declare do_photons=0;
    #declare do_reflected_blocks=1;
    #declare do_spotlights=0;
    #declare do_extra_blocks=1;
    #declare do_pyramid_blocks=1;
    #declare do_mirror=1;
    #declare do_close_camera=0;
    #declare do_low_light=0;
    #declare do_main_light=1;
    #declare do_board_floor=1;
    #declare do_floor_photons=0;
    #declare save_photons=0;
    #declare disable_photon_refraction=0;
    #declare save_rad=0;
    #declare simple_blocks=0;
    #declare simple_textures=0;
    #declare no_glass=0;
    #declare use_photons=1;
    #declare do_extra_lights=0;
    #declare do_sun=1;
    #declare point_sun=1;
    #declare sun_power=3;
    #declare sky_brightness=7;
    #declare light_level=0.9;
#end

/* ************************
 * Trace number 8 renders the scene without
 * radiosity or photons. No pretraces are
 * necessary for this setting.
 */

#if (final_trace_number = 8)
    #declare do_focal_blur=0;
    #declare do_radiosity=0;
    #declare do_area_light=0;
    #declare do_open_window=0;
    #declare do_photons=0;
    #declare do_reflected_blocks=1;
    #declare do_spotlights=0;
    #declare do_extra_blocks=1;
    #declare do_pyramid_blocks=1;
    #declare do_mirror=1;
    #declare do_close_camera=0;
    #declare do_low_light=0;
    #declare do_main_light=1;
    #declare do_board_floor=1;
    #declare do_floor_photons=0;
    #declare save_photons=0;
    #declare disable_photon_refraction=0;
    #declare save_rad=0;
    #declare simple_blocks=1;
    #declare simple_textures=1;
    #declare no_glass=1;
    #declare use_photons=1;
    #declare do_extra_lights=0;
    #declare do_sun=1;
    #declare point_sun=1;
    #declare sun_power=3;
    #declare sky_brightness=7;
    #declare light_level=0.9;
#end

global_settings {
    //assumed_gamma 1.0
    max_trace_level 100
    adc_bailout 0.05
    #if (do_radiosity)

        /* ********************
         * Boy, I really don't understand radiosity
         * very well. The following settings I have
         * gleaned from a combination of multiple sources
         * and also represent a bit of my own experimentation.
         * Don't ask me to explain exactly why they work.
         * I just know that the results are pretty good.
         */
        radiosity {
            #if (quick_radiosity)
            #if (save_rad)
                brightness 1  // Will be lowered in the load step
                count 500  // Does it need to be this high?
                            // It's not overly slow, so.... why not?
                error_bound 2  // I'd like this to be lower, but then the
                               // radiosity step takes FOREVER to prodcess.
                gray_threshold 0.0  // Hey! I actually understand this one!
                low_error_factor 0.6  // Hmmm... Must be magic.
                minimum_reuse 0.0025  // Again, no clue.
                nearest_count 10  // The maximum value.
                recursion_limit 1 // Very confusing parameter. I leave it alone.

                media            off // I'm not using media, so doesn't matter.
                normal           off  // Yes, I AM normal, thank you very much.
                always_sample    on  // I think that's the point of this pass.

                pretrace_start   0.08 // Yeilds three passes. I expected four??
                pretrace_end     0.01
                save_file "block.rad"
            #else
                pretrace_start 1 pretrace_end 1  // No pretrace.
                load_file "block.rad"
                brightness 1  // Please don't over-saturate my scene.
                error_bound 8   // Multiply it by four. Why? Not sure!
                recursion_limit 1  // Still very confusing.
                always_sample off  // Use saved samples, whenever possible.
            #end
            #else
            #if (save_rad)
                brightness 1  // May be lowered in the load step
                count 1600  // Does it need to be this high?
                            // It's not overly slow, so.... why not?
                error_bound 0.1  // I'd think this should be low enough!
                gray_threshold 0.0  // Hey! I actually understand this one!
                low_error_factor 0.7  // Hmmm... Must be magic.
                minimum_reuse 0.01  // Again, no clue.
                nearest_count 8  // The maximum value.
                recursion_limit 1 // Very confusing parameter. I leave it alone.
                adc_bailout 0.01/2

                media            off // I'm not using media, so doesn't matter.
                normal           off // I've never been normal.
                always_sample    on // Theoretically, not necessary

                pretrace_start   0.08 // Four? passes
                pretrace_end     0.004
                save_file "block.rad"
            #else
                brightness 1  // For now, use the same as above
                pretrace_start 1 pretrace_end 1  // No pretrace.
                error_bound 0.4  // Multiply by four for some unknown reason
                load_file "block.rad"
                recursion_limit 1 // Still very confusing.
                always_sample off // Use saved values
            #end
            #end
        }
    #end
    #if (do_photons)
        photons {
            #if (save_photons)
                spacing 0.005
                radius 0.025

                autostop 0    // Not sure if this is needed?

                #if (final_trace_number < 3)
                    save_file "block.pho1"
                #else
                    save_file "block.pho2"
                #end
            #else
                #if (final_trace_number < 3 & final_trace_number != 0)
                    load_file "block.pho1"
                #else
                    #if (final_trace_number = 6)
                        load_file "block.pho1"
                    #else
                        load_file "block.pho2"
                    #end
                #end
            #end
        }
    #end
}


#if (do_close_camera)
camera {
  location  <0, 0.8, -3>
  look_at   <0, 1,  0>
  right     x*image_width/image_height
}
object {
    wavy_mirror_block()
    rotate y*30
    translate <0, block_size, 0> 
}
#else
camera {
  location  <4.5, 5.5, -7>
  look_at   <0.5, 2.5,  0> 
  right     x*image_width/image_height
  #if (do_focal_blur)
      aperture 0.05 
      blur_samples 100
      variance 1/2000
      confidence 0.95
      focal_point <0, 5, -1>
  #end
}
#end




#if (do_low_light)
light_source {
    <-10, 1.5, -30>
    color rgb <.6, .6, .6>
}
#end

#declare left_wall = -50;
#declare right_wall = 50;
#declare ceiling = 52;
#declare back_wall = -100;
#declare front_wall = 7;

#if (! do_open_window)
    #if (do_area_light)
        light_source {
            0*x                  // light's position (translated below)
            color rgb <0.8,0.8,0.9>    // light's color
            area_light
            y,z,50,20
            adaptive 2
            fade_distance 50
            fade_power 2
            translate <right_wall-0.001, 25, -35>
            photons {
                #if (disable_photon_refraction)
                    refraction off
                #else
                    refraction on
                #end
                reflection on
            }
        }
    #else
        light_source {
            0*x                  // light's position (translated below)
            color rgb <0.9,0.9,1.0>*light_level * 0.8
            fade_distance 50
            fade_power 2
            translate <right_wall-0.001, 28, front_wall-25>
        }
        light_source { 0*x                  // light's position (translated below)
            color rgb <0.9,0.9,1.0>*light_level * 0.8
            fade_distance 50
            fade_power 2
            translate <right_wall-0.001, 28, back_wall+25>
        }
    #end
#end

#if (do_spotlights)
    #declare spot_intensity=0.2;
    light_source {
        0*x
        color rgb spot_intensity
        spotlight
        radius 10
        falloff 20
        tightness 5
        point_at <0.05*block_size, block_size*5, -0.02>
        shadowless
        translate <4, 3, -3>
        photons {
            reflection off
            refraction off
        }
    }
    light_source {
        0*x
        color rgb spot_intensity
        spotlight
        radius 10
        falloff 20
        tightness 5
        point_at <0.05*block_size, block_size*5, -0.02>
        shadowless
        translate <-4, 3, -3>
        photons {
            reflection off
            refraction off
        }
    }
    light_source {
        0*x
        color rgb spot_intensity
        spotlight
        radius 30
        falloff 20
        tightness 10
        point_at <0.05*block_size, block_size*3, -0.02>
        shadowless
        translate <0, 2, -3>
        photons {
            reflection off
            refraction off
        }
    }
#end

#if (do_main_light)
    light_source {
        0*x                  // light's position (translated below)
        color rgb <1.0,0.98,0.85>*light_level    // light's color
        fade_distance 50
        fade_power 2
        translate <10, ceiling-2, -45>
        photons {
            #if (disable_photon_refraction)
                refraction off
            #else
                refraction on
            #end
            reflection on
        }
    }
#end

#if (do_extra_lights)
    light_source {
        0*x                  // light's position (translated below)
        color rgb <0.68,0.62,0.60>*light_level    // light's color
        fade_distance 50
        fade_power 2
        translate <14, ceiling-2, -20>
        photons {
            #if (disable_photon_refraction)
                refraction off
            #else
                refraction on
            #end
            reflection on
        }
    }
    light_source {
        0*x                  // light's position (translated below)
        color rgb <0.80,0.73,0.70>*light_level    // light's color
        fade_distance 50
        fade_power 2
        translate <12, ceiling-2, -18>
        photons {
            #if (disable_photon_refraction)
                refraction off
            #else
                refraction on
            #end
            reflection on
        }
    }
#end

#declare wall_texture =
texture {
    pigment { color rgb <1.0, 1.0, 1.0> }
    finish {
        diffuse 0.7
        reflection 0
    } 
}

box {
    <left_wall,0,front_wall>
    <right_wall,ceiling,front_wall+1>
    texture { wall_texture }
}

cylinder {
    <left_wall,0,front_wall>
    <right_wall,0,front_wall>
    0.3
    texture { wall_texture }
}

#declare sun_light =
#if (do_sun)
    #if (point_sun)
        light_source {
            0*x                  // light's position (translated below)
            color rgb <0.90,0.92,1.0>*sun_power    // light's color
            translate <1800, 700, -50>
            photons {
                #if (disable_photon_refraction)
                    refraction off
                #else
                    refraction on
                #end
                reflection on
            }
        }
    #end
#end

#if (do_open_window)
    union {
        difference {
            box {
                <right_wall,0,front_wall>
                <right_wall+1,ceiling,back_wall>
            }
            box {
                <right_wall-1, 10, front_wall-10>
                <right_wall+2, 45, front_wall-40>
            }
            box {
                <right_wall-1, 10, back_wall+10>
                <right_wall+2, 45, back_wall+40>
            }
        }
        box {
            <right_wall,10,front_wall-24>
            <right_wall+0.5,45,front_wall-26>
        }
        box {
            <right_wall,26,front_wall-10>
            <right_wall+0.5,29,front_wall-40>
        }
        box {
            <right_wall,10,back_wall+24>
            <right_wall+0.5,45,back_wall+26>
        }
        box {
            <right_wall,26,back_wall+10>
            <right_wall+0.5,29,back_wall+40>
        }
        texture { wall_texture }
    }

    sphere {
        <0,0,0>
        2000
        hollow
        pigment { color rgb <0.75, 0.85, 1> }
        finish {
            diffuse 0
            ambient sky_brightness
        }
    }
    #if (do_photons & save_photons)
        light_source {
            0*x
            color rgb <0.65, 0.75, 1>
            translate <right_wall, 27.5, front_wall-25>
            photons {
                #if (disable_photon_refraction)
                    refraction off
                #else
                    refraction on
                #end
                reflection on
            }
        }
        light_source {
            0*x
            color rgb <0.65, 0.75, 1>
            translate <right_wall, 27.5, back_wall+25>
            photons {
                #if (disable_photon_refraction)
                    refraction off
                #else
                    refraction on
                #end
                reflection on
            }
        }
    #end

    sun_light

#else
    box {
        <right_wall,0,front_wall>
        <right_wall+1,ceiling,back_wall>
        texture { wall_texture }
    }
#end

box {
    <left_wall,0,front_wall>
    <left_wall-1,ceiling,back_wall>
    texture { wall_texture }
}

box {
    <left_wall,0,back_wall>
    <right_wall,ceiling,back_wall-1>
    texture { wall_texture }
}

box {
    <left_wall,ceiling,back_wall>
    <right_wall,ceiling+1,front_wall>
    texture { wall_texture }
}


#if (do_board_floor)
object {
    wood_floor(0.005, 2, 14, 0, -left_wall, -back_wall)
}
box {
    <left_wall,-2,back_wall>
    <right_wall,-1,7>
    pigment { color rgb 0 }
}
#else

box {
    <left_wall,-1,back_wall>
    <right_wall,0,7>
    wood_floor_texture(1,1,1)
}
light_group {
    sun_light
    box {
        <left_wall,-1,back_wall>
        <right_wall,0,7>
        wood_floor_texture(1,1,1)
        #if (do_floor_photons)
            photons { target reflection on refraction off }
        #end
    }
}
#end

#if (do_mirror)

merge {
    box {
        <-3, 0, -0.05> 
        <4, 6, -0.009>
        texture {
            finish {
                specular 0.2
                roughness 0.02
                ambient 0
                diffuse 0.02
                reflection {
                    0.08, 0.6
                    fresnel on
                }
                conserve_energy
            }
            pigment { Clear }
        }
        interior {
            ior 1.7
            fade_distance 1.0
            fade_power 2
        }
        photons {
            target 2
            reflection on
            refraction on
            collect off
        }
    } 
    box {
        <-3, 0, 0> 
        <4, 6, -0.01>
        texture {
            pigment { White }
            finish {
                specular 0
                roughness 0.0001
                ambient 0
                diffuse 0.08
                reflection 1 
            }
        }
        photons {
            target 2
            reflection on
            collect off
        }
    }
    rotate x*asind(0.3/6)
    translate <0, 0.001, 6.699>
}

#end

#if (do_pyramid_blocks)

object {
    two_gear_block(0)
    rotate y*-2
    translate <0.03, block_size, 0.02> 
}
object {
    circus_cylinder_block(0)
    rotate y*2
    translate <-2.1*block_size, block_size, -0.1>
} 
object {
    stripey_cylinder_block(-35)
    rotate y*3
    translate <2.2*block_size, block_size, -0.07>
}
object {
    smiley_block(90)
    rotate y*-2
    translate <-1.03*block_size, block_size*3, 0.02>
} 
object {
    wavy_mirror_block()
    rotate y*3
    translate <1.2*block_size, block_size*3, 0.03>
}
object {
    balloon_block()
    translate <0.05*block_size, block_size*5, -0.02>
    rotate y*-8
}

#end

#if (do_extra_blocks)

object {
    balloon_block()
    rotate x*-90
    rotate y*-105
    translate <-8.7, block_size, 1>
}

object {
    triangular_prism_block(90)
    rotate y*-90
    rotate x*-90
    rotate y*53
    translate <-4, block_size, -2>
}
object {
    smiley_block(5)
    rotate x*90
    rotate y*-145
    translate <0, block_size, -5>
}
object {
    wavy_mirror_block()
    rotate x*-90
    rotate y*-160
    translate <-9.1, block_size, 3.2>
}
object {
    circus_cylinder_block(0)
    rotate x*180
    rotate y*50
    translate <4, block_size, 3.14>
}
object {
    bell_block()
    rotate x*90
    rotate y*-65
    translate <4.25, block_size*3, 3.4>
}
object {
    two_gear_block(50)
    rotate y*-90
    rotate x*90
    rotate y*168
    translate <4, block_size, -3>
}
object {
    bell_block()
    rotate x*90
    rotate y*57
    translate <-9.3, block_size*3, 2.2>
}
object {
    wavy_mirror_block()
    rotate x*180
    rotate y*90
    translate <5.6, block_size, 0.5>
}
object {
    triangular_prism_block(80)
    rotate y*210
    translate <5.86, block_size*3, 0.67>
}
object {
    povray_block(0)
    rotate y*20
    translate <6.5, block_size, -3>
}

#end

#if (do_reflected_blocks)

#if (final_trace_number = 9)
    #declare no_glass=1;
    #declare simple_textures=1;
#end

#declare use_photons=0;
#declare glass_disabled = 0;

/*#declare right_cut_off = 6;
#declare left_cut_off = left_wall+1.0001+block_size/4;
#declare min_blocks = 5;
#declare horiz_move = 6;
#declare vert_move = 3;
#declare max_blocks = 10;
//#declare max_blocks = (right_cut_off - left_cut_off) / horiz_move;
#declare vert_start = -6;
#declare max_height = 15;
#declare height_slope = 2.8;*/

#declare right_cut_off = 10;
#declare left_cut_off = left_wall;
#declare min_blocks = 3;
#declare max_blocks = 8;
#declare vert_move = 5;
#declare vert_start = -6;
#declare max_height = 8;
#declare height_slope = 0.8;


#declare total_blocks = 0;
#declare skip_blocks = 0;
#declare with_photons = 0;

//#declare sd=seed(4);
#declare sd=seed(7);
#declare sd2=seed(20);


#declare i=0;
#while (i < (vert_start-back_wall / vert_move) - 1)
    #declare row_max_height = 0;
    #declare row_skipped = 0;
    #declare row_blocks = 0;
    #declare j=0;
    #declare num_blocks=int(rand(sd) * (max_blocks-min_blocks)) + min_blocks;
    #declare horiz_spacing = right_cut_off;
    #while (j < num_blocks)
        #declare horiz_spacing = horiz_spacing - (right_cut_off-left_cut_off)/(num_blocks+1);
        #declare vert_spacing = rand(sd) * 2 - 1;
        #declare this_max_height = min(i/height_slope, max_height);
        //#declare height = int(rand(sd) * this_max_height) + 1;
        #declare height = min(int(rand(sd) * i/height_slope) + 1, max_height);
        #if (height > row_max_height)
            #declare row_max_height = height;
        #end
            #declare k=0;
            #while (k < height)
                #declare block_type=int(rand(sd2) * 9);
                #declare x_rot = int(rand(sd) * 3);
                #declare y_rot = rand(sd) * 360;
                #declare x_jitter = rand(sd) * block_size/2 - block_size/4;
                #declare z_jitter = rand(sd) * block_size/2 - block_size/4;
                #declare item_rot = rand(sd) * 360;
                #declare vert_pos = vert_start - (i * vert_move) +
                                   vert_spacing + z_jitter;
                /*#if (k <= this_max_height - 7 | k > 11 | i < 2)
                #declare skip_blocks = skip_blocks + 1;
                #declare row_skipped = row_skipped + 1;
                #else*/
                #declare row_blocks = row_blocks + 1;
                #declare total_blocks = total_blocks + 1;
                /*#if (k > min(height,11) - 2 &
                     k >= min(this_max_height,12) - 3)
                    #declare use_photons=2;
                    #declare with_photons = with_photons + 1;
                #else
                    #declare use_photons=0;
                    #if (do_photons & save_photons)
                        #declare glass_disabled = 1;
                        #declare save_no_glass = no_glass;
                        #declare no_glass = 1;
                    #end
                #end*/
                object
                {
                    #switch (block_type)
                        #case (0)
                        stripey_cylinder_block(item_rot)
                        #break
                        #case (1)
                        #if (! glass_disabled & do_photons & save_photons)
                            #declare glass_disabled = 1;
                            #declare save_no_glass = no_glass;
                            #declare no_glass = 1;
                        #end
                        wavy_mirror_block()
                        #break
                        #case (2)
                        circus_cylinder_block(item_rot)
                        #break
                        #case (3)
                        balloon_block()
                        #break
                        #case (4)
                        bell_block()
                        #break
                        #case (5)
                        smiley_block(item_rot)
                        #break
                        #case (6)
                        triangular_prism_block(item_rot)
                        #break
                        #case (7)
                        two_gear_block(item_rot)
                        #break
                        #case (8)
                        povray_block(item_rot)
                        #break
                    #end
                    #switch (x_rot)
                        #case (1)
                        rotate x*90
                        #case (2)
                        rotate x*180
                    #end
                    rotate y*y_rot
                    translate <horiz_spacing + x_jitter,
                               block_size * (k * 2 + 1),
                               vert_pos>
                }
                #if (glass_disabled)
                    #declare glass_disabled = 0;
                    #declare no_glass = save_no_glass;
                #end
                //#end
                #declare k=k+1;
            #end
        #declare j=j+1;
    #end
    #debug "** Row "
    #debug str(i, 0, 0)
    #debug ": "
    #debug str(row_blocks, 0, 0)
    #debug " blocks in "
    #debug str(num_blocks, 0, 0)
    #debug " stacks, max height: "
    #debug str(this_max_height, 0, 0)
    #debug ", actual max: "
    #debug str(row_max_height, 0, 0)
    #debug ", skipped blocks: "
    #debug str(row_skipped, 0, 0)
    #debug "\n"
    #declare i=i+1;
#end
#debug "\n\n** Total blocks: "
#debug str(total_blocks, 0, 0)
#debug "\n** Photon blocks: "
#debug str(with_photons, 0, 0)
#debug "\n** Skipped blocks: "
#debug str(skip_blocks, 0, 0)
#debug "\n"

#end
