#include "math.inc"
#include "colors.inc"
#include "logo.inc"

#declare use_photons=1;

#declare box_size = 1.0;
#declare wall_thickness = 0.04*box_size;
#declare corner_width = 0.3*box_size; 
#declare small_corner_width = 0.1*box_size;
#declare cap_height = 3*(corner_width - small_corner_width)/2;
#declare glass_height = box_size - cap_height - small_corner_width;
#declare cylinder_size = 0.8*box_size;

#declare block_glass_texture =
texture {
    finish {
        //specular 1
        //roughness 0.02
        ambient 0
        diffuse 0
        reflection {
            0.02, 0.9
            fresnel on
        }
        conserve_energy
    }
    pigment { Clear }
}
#declare block_glass_interior =
interior {
    ior 1.7
    fade_distance 1.0
    fade_power 2
}

#declare clear_round_corner_box = 
intersection {   
    merge {
        // Front left corner
        cylinder {
          <-box_size + corner_width, -2*box_size, -box_size + corner_width>
          <-box_size + corner_width, 2*box_size, -box_size + corner_width>
          corner_width
        }
        // Front right corner
        cylinder {
          <box_size - corner_width, -2*box_size, -box_size + corner_width>
          <box_size - corner_width, 2*box_size, -box_size + corner_width>
          corner_width
        }
        // Back left corner
        cylinder {
          <-box_size + corner_width, -2*box_size, box_size - corner_width>
          <-box_size + corner_width, 2*box_size, box_size - corner_width>
          corner_width
        }
        // Back right corner
        cylinder {
          <box_size - corner_width, -2*box_size, box_size - corner_width>
          <box_size - corner_width, 2*box_size, box_size - corner_width>
          corner_width
        }
        box {
          <-box_size + corner_width, -2*box_size, -box_size>
          <box_size - corner_width, 2*box_size,  box_size>
        }            
        box {
          <-box_size, -2*box_size, -box_size + corner_width>
          <box_size, 2*box_size,  box_size - corner_width>
        }
    }
    box {
        <-box_size-0.001, -box_size, -box_size-0.001>
        <box_size+0.001, box_size, box_size+0.001>
    }
}

#declare round_corner_box = 
intersection {   
    union {
        // Front left corner
        cylinder {
          <-box_size + corner_width, -2*box_size, -box_size + corner_width>
          <-box_size + corner_width, 2*box_size, -box_size + corner_width>
          corner_width
        }
        // Front right corner
        cylinder {
          <box_size - corner_width, -2*box_size, -box_size + corner_width>
          <box_size - corner_width, 2*box_size, -box_size + corner_width>
          corner_width
        }
        // Back left corner
        cylinder {
          <-box_size + corner_width, -2*box_size, box_size - corner_width>
          <-box_size + corner_width, 2*box_size, box_size - corner_width>
          corner_width
        }
        // Back right corner
        cylinder {
          <box_size - corner_width, -2*box_size, box_size - corner_width>
          <box_size - corner_width, 2*box_size, box_size - corner_width>
          corner_width
        }
        box {
          <-box_size + corner_width, -2*box_size, -box_size>
          <box_size - corner_width, 2*box_size,  box_size>
        }            
        box {
          <-box_size, -2*box_size, -box_size + corner_width>
          <box_size, 2*box_size,  box_size - corner_width>
        }
        split_union off
    }
    box {
        <-box_size-0.001, -box_size, -box_size-0.001>
        <box_size+0.001, box_size, box_size+0.001>
    }
}

#macro glass_walls()
    difference {
        object {
            clear_round_corner_box
            scale <1, glass_height*0.9999/box_size, 1>
        }
        object {
            round_corner_box
            scale <(box_size-wall_thickness)/box_size, 2, (box_size-wall_thickness)/box_size>
        }
        texture { block_glass_texture }
        interior { block_glass_interior }
        #if (use_photons = 0)
            photons {
                pass_through
                collect off
            }
        #else
            photons {
                target use_photons
                refraction on
                reflection on
                collect off
            }
        #end
    }
#end


#declare block_cover_base = 
object {
    round_corner_box
    scale <1, (cap_height/6)/box_size, 1>
    translate y*(cap_height/6)
} 


#declare block_cover_lower_corner = 
union {
    // Left side
    cylinder {
        <-box_size + small_corner_width, cap_height/3, -box_size + corner_width>
        <-box_size + small_corner_width, cap_height/3, box_size - corner_width>
        small_corner_width
    }  
    // Right side
    cylinder {
        <box_size - small_corner_width, cap_height/3, -box_size + corner_width>
        <box_size - small_corner_width, cap_height/3, box_size - corner_width>
        small_corner_width
    }              
    // Front
    cylinder {
        <-box_size + corner_width, cap_height/3, -box_size + small_corner_width>
        <box_size - corner_width, cap_height/3, -box_size + small_corner_width>
        small_corner_width
    }       
    //Back
    cylinder {
        <-box_size + corner_width, cap_height/3, box_size - small_corner_width>
        <box_size - corner_width, cap_height/3, box_size - small_corner_width>
        small_corner_width
    }        
    // Front left corner
    union {
        torus {
            corner_width - small_corner_width,
            small_corner_width
            translate <-box_size + corner_width, cap_height/3, -box_size + corner_width>
        }
        cylinder {
            <-box_size + corner_width, cap_height/3-small_corner_width, -box_size + corner_width>  
            <-box_size + corner_width, cap_height/3+small_corner_width, -box_size + corner_width>
            corner_width-small_corner_width
        }
        split_union off
    }  
    // Front right corner
    union {
        torus {
            corner_width - small_corner_width,
            small_corner_width
            translate <box_size - corner_width, cap_height/3, -box_size + corner_width>
        }
        cylinder {
            <box_size - corner_width, cap_height/3-small_corner_width, -box_size + corner_width>  
            <box_size - corner_width, cap_height/3+small_corner_width, -box_size + corner_width>
            corner_width-small_corner_width
        }
        split_union off
    }
    // Back left corner
    union {
        torus {
            corner_width - small_corner_width,
            small_corner_width
            translate <-box_size + corner_width, cap_height/3, box_size - corner_width>
        }
        cylinder {
            <-box_size + corner_width, cap_height/3-small_corner_width, box_size - corner_width>  
            <-box_size + corner_width, cap_height/3+small_corner_width, box_size - corner_width>
            corner_width-small_corner_width
        }
        split_union off
    }
    // Back right corner
    union {
        torus {
            corner_width - small_corner_width,
            small_corner_width
            translate <box_size - corner_width, cap_height/3, box_size - corner_width>
        }
        cylinder {
            <box_size - corner_width, cap_height/3-small_corner_width, box_size - corner_width>  
            <box_size - corner_width, cap_height/3+small_corner_width, box_size - corner_width>
            corner_width-small_corner_width
        }
        split_union off
    }
    split_union off
}

#declare block_cover_slant_walls =
difference {
    union {
        // Front left corner
        cone {
            <-box_size + small_corner_width*(1-sind(45)) + corner_width-small_corner_width*(1-sind(45)), cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + corner_width-small_corner_width*(1-sind(45))>,  corner_width-small_corner_width*(1-sind(45)),
            <-box_size + small_corner_width*(1-sind(45)) + corner_width-small_corner_width*(1-sind(45)), cap_height + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + corner_width-small_corner_width*(1-sind(45))>, small_corner_width-small_corner_width*(1-sind(45))
        }
        // Front right corner
        cone {
            <box_size - small_corner_width*(1-sind(45)) - corner_width+small_corner_width*(1-sind(45)), cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + corner_width-small_corner_width*(1-sind(45))>,  corner_width-small_corner_width*(1-sind(45)),
            <box_size - small_corner_width*(1-sind(45)) - corner_width+small_corner_width*(1-sind(45)), cap_height + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + corner_width-small_corner_width*(1-sind(45))>, small_corner_width-small_corner_width*(1-sind(45))
        }
        // Back left corner
        cone {
            <-box_size + small_corner_width*(1-sind(45)) + corner_width-small_corner_width*(1-sind(45)), cap_height/3 + small_corner_width*sind(45), box_size - small_corner_width*(1-sind(45)) - corner_width+small_corner_width*(1-sind(45))>,  corner_width-small_corner_width*(1-sind(45)),
            <-box_size + small_corner_width*(1-sind(45)) + corner_width-small_corner_width*(1-sind(45)), cap_height + small_corner_width*sind(45), box_size - small_corner_width*(1-sind(45)) - corner_width+small_corner_width*(1-sind(45))>, small_corner_width-small_corner_width*(1-sind(45))
        }
        // Back right corner
        cone {
            <box_size - small_corner_width*(1-sind(45)) - corner_width+small_corner_width*(1-sind(45)), cap_height/3 + small_corner_width*sind(45), box_size - small_corner_width*(1-sind(45)) - corner_width+small_corner_width*(1-sind(45))>,  corner_width-small_corner_width*(1-sind(45)),
            <box_size - small_corner_width*(1-sind(45)) - corner_width+small_corner_width*(1-sind(45)), cap_height + small_corner_width*sind(45), box_size - small_corner_width*(1-sind(45)) - corner_width+small_corner_width*(1-sind(45))>, small_corner_width-small_corner_width*(1-sind(45))
        }
        // Left
        box {
            <-sqrt(2)*cap_height/3, -box_size, -box_size + corner_width>
            <sqrt(2)*cap_height/3, wall_thickness/2, box_size - corner_width>
            rotate z*45
            translate <-box_size + small_corner_width*(1-sind(45)) + cap_height/3 + wall_thickness*sind(45)/2, 2*cap_height/3 + small_corner_width*sind(45) - wall_thickness*sind(45)/2, 0>
     
        }
        // Right
        box {
            <-sqrt(2)*cap_height/3, -box_size, -box_size + corner_width>
            <sqrt(2)*cap_height/3, wall_thickness/2, box_size - corner_width>
            rotate z*-45
            translate <box_size - small_corner_width*(1-sind(45)) - cap_height/3 - wall_thickness*sind(45)/2, 2*cap_height/3 + small_corner_width*sind(45) - wall_thickness*sind(45)/2, 0>
     
        }
        // Front
        box {
            <-box_size + corner_width, -box_size, -sqrt(2)*cap_height/3>
            <box_size - corner_width, wall_thickness/2, sqrt(2)*cap_height/3>
            rotate x*-45
            translate <0, 2*cap_height/3 + small_corner_width*sind(45) - wall_thickness*sind(45)/2, -box_size + small_corner_width*(1-sind(45)) + cap_height/3 + wall_thickness*sind(45)/2>
     
        }
        // Back
        box {
            <-box_size + corner_width, -box_size, -sqrt(2)*cap_height/3>
            <box_size - corner_width, wall_thickness/2, sqrt(2)*cap_height/3>
            rotate x*45
            translate <0, 2*cap_height/3 + small_corner_width*sind(45) - wall_thickness*sind(45)/2, box_size - small_corner_width*(1-sind(45)) - cap_height/3 - wall_thickness*sind(45)/2>
     
        }
        split_union off
    }
    box {
        <-box_size, -box_size, -box_size>
        <box_size, cap_height/3-0.001, box_size>
    }
}  

#declare block_cover_upper_corner = 
union {
    // Left side
    cylinder {
        <-box_size + 2*cap_height/3 + small_corner_width, cap_height, -box_size + 2*cap_height/3 + small_corner_width>
        <-box_size + 2*cap_height/3 + small_corner_width, cap_height, box_size - 2*cap_height/3 - small_corner_width>
        small_corner_width
    }  
    // Left side
    cylinder {
        <box_size - 2*cap_height/3 - small_corner_width, cap_height, -box_size + 2*cap_height/3 + small_corner_width>
        <box_size - 2*cap_height/3 - small_corner_width, cap_height, box_size - 2*cap_height/3 - small_corner_width>
        small_corner_width
    }  
    // Front
    cylinder {
        <box_size - 2*cap_height/3 - small_corner_width, cap_height, -box_size + 2*cap_height/3 + small_corner_width>
        <-box_size + 2*cap_height/3 + small_corner_width, cap_height, -box_size + 2*cap_height/3 + small_corner_width>
        small_corner_width
    }  
    //Back
    cylinder {
        <box_size - 2*cap_height/3 - small_corner_width, cap_height, box_size - 2*cap_height/3 - small_corner_width>
        <-box_size + 2*cap_height/3 + small_corner_width, cap_height, box_size - 2*cap_height/3 - small_corner_width>
        small_corner_width
    }  
    // Front left corner
    sphere {
        <-box_size + 2*cap_height/3 + small_corner_width, cap_height, -box_size + 2*cap_height/3 + small_corner_width>
        small_corner_width
    }
    // Front right corner
    sphere {
        <box_size - 2*cap_height/3 - small_corner_width, cap_height, -box_size + 2*cap_height/3 + small_corner_width>
        small_corner_width
    }
    // Back left corner
    sphere {
        <-box_size + 2*cap_height/3 + small_corner_width, cap_height, box_size - 2*cap_height/3 - small_corner_width>
        small_corner_width
    }
    // Back right corner
    sphere {
        <box_size - 2*cap_height/3 - small_corner_width, cap_height, box_size - 2*cap_height/3 - small_corner_width>
        small_corner_width
    }
    // Top
    box {
        <box_size - 2*cap_height/3 - small_corner_width, cap_height + small_corner_width, box_size - 2*cap_height/3 - small_corner_width>
        <-box_size + 2*cap_height/3 + small_corner_width, 0.001, -box_size + 2*cap_height/3 + small_corner_width>
    }       
    split_union off
}

#declare block_cover =
difference {
    union {
        object { block_cover_base }
        object { block_cover_lower_corner }
        object { block_cover_slant_walls }
        object { block_cover_upper_corner }
        split_union off
    } 
    object {
        round_corner_box
        scale <box_size-wall_thickness, cap_height/2, box_size-wall_thickness>
    }
}

#declare rough_block_cover =
difference {
    box {
        <-box_size, 0, -box_size>
        <box_size, cap_height+small_corner_width, box_size>
    }
    box {
        <-box_size*2, 0, 0>
        <box_size*2, box_size, box_size>
        rotate x*-45
        translate z*(-box_size-cap_height/3)
        rotate y*0
    }
    box {
        <-box_size*2, 0, 0>
        <box_size*2, box_size, box_size>
        rotate x*-45
        translate z*(-box_size-cap_height/3)
        rotate y*90
    }
    box {
        <-box_size*2, 0, 0>
        <box_size*2, box_size, box_size>
        rotate x*-45
        translate z*(-box_size-cap_height/3)
        rotate y*180
    }
    box {
        <-box_size*2, 0, 0>
        <box_size*2, box_size, box_size>
        rotate x*-45
        translate z*(-box_size-cap_height/3)
        rotate y*-90
    }
}

#declare shallow_indent = 0.01*box_size;
#declare deeper_indent = 0.07*box_size;
#declare logo_width = 0.8*box_size;
#declare logo_circle_height = (sqrt(2)*cap_height/3)*0.3;
#declare logo_circle_size = sqrt( pow(sqrt(2)*cap_height/3-logo_circle_height, 2) + pow(logo_width/8, 2) );             
           

  
#declare top_text_logo =
difference {
    union {
        box {
            <-logo_width/2, -shallow_indent, -sqrt(2)*cap_height/3+logo_circle_height>
            <logo_width/2, shallow_indent, sqrt(2)*cap_height/3> 
        } 
        intersection {
            union {
                cylinder {
                    <-3*logo_width/8, -shallow_indent, 0>
                    <-3*logo_width/8, shallow_indent, 0>
                    logo_circle_size
                }    
                cylinder {
                    <-logo_width/8, -shallow_indent, 0>
                    <-logo_width/8, shallow_indent, 0>
                    logo_circle_size
                }   
                cylinder {
                    <logo_width/8, -shallow_indent, 0>
                    <logo_width/8, shallow_indent, 0>
                    logo_circle_size
                }  
                cylinder {
                    <3*logo_width/8, -shallow_indent, 0>
                    <3*logo_width/8, shallow_indent, 0>
                    logo_circle_size
                }
            }
            box {
                <-logo_width/2, -shallow_indent - 0.001, -sqrt(2)*cap_height/3>
                <logo_width/2, shallow_indent + 0.001, sqrt(2)*cap_height/3>
            }
        }     
    }
    text {
      ttf
      "Jester.ttf", "Fisher-Price",
      0.008,0
      scale <0.14, 0.15, 1>
      rotate x*90
      translate <-logo_width/2+0.04, -shallow_indent+0.004, -cap_height*0.07>
    }  
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}

#declare top_POV_logo =
difference {
    union {
        box {
            <-logo_width/2, -shallow_indent, -sqrt(2)*cap_height/3+logo_circle_height>
            <logo_width/2, shallow_indent, sqrt(2)*cap_height/3> 
        } 
        intersection {
            union {
                cylinder {
                    <-3*logo_width/8, -shallow_indent, 0>
                    <-3*logo_width/8, shallow_indent, 0>
                    logo_circle_size
                }    
                cylinder {
                    <-logo_width/8, -shallow_indent, 0>
                    <-logo_width/8, shallow_indent, 0>
                    logo_circle_size
                }   
                cylinder {
                    <logo_width/8, -shallow_indent, 0>
                    <logo_width/8, shallow_indent, 0>
                    logo_circle_size
                }  
                cylinder {
                    <3*logo_width/8, -shallow_indent, 0>
                    <3*logo_width/8, shallow_indent, 0>
                    logo_circle_size
                }
            }
            box {
                <-logo_width/2, -shallow_indent - 0.001, -sqrt(2)*cap_height/3>
                <logo_width/2, shallow_indent + 0.001, sqrt(2)*cap_height/3>
            }
        }     
    }
    text {
      ttf
      "Jester.ttf", "POV-Block",
      0.008,0
      scale <0.14, 0.15, 1>
      rotate x*90
      translate <-logo_width/2+0.04, -shallow_indent+0.004, -cap_height*0.07>
    }  
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}


#declare front_dot_size = sqrt(2)*cap_height/9;

#declare top_front_dots_cluster =
union {
    sphere {
        <0, 0, 0>
        front_dot_size
    }
    sphere {
        <-front_dot_size*2, 0, -front_dot_size*2>
        front_dot_size
    }
    sphere {
        <-front_dot_size*2, 0, front_dot_size*2>
        front_dot_size
    }
    sphere {
        <front_dot_size*2, 0, front_dot_size*2>
        front_dot_size
    }
    sphere {
        <front_dot_size*2, 0, -front_dot_size*2>
        front_dot_size
    }
    translate z*-front_dot_size/2
}

#declare top_front_dots =
union {
    object {
        top_front_dots_cluster
        rotate x*-45
        translate <-box_size + corner_width+3*front_dot_size, 2*cap_height/3 + small_corner_width*sind(45)+front_dot_size - shallow_indent, -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
    }
    object {
        top_front_dots_cluster
        rotate x*-45
        translate <box_size - corner_width-3*front_dot_size, 2*cap_height/3 + small_corner_width*sind(45)+front_dot_size - shallow_indent, -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
    }
}

#declare top_dot_size = (box_size*2 - 2*corner_width)/34;
#declare top_box_size = (box_size*2 - 2*corner_width)/6.5;
#declare top_box_space = (box_size*2 - 2*corner_width - top_box_size*5)/6;

#declare top_front_large_stripes =
union {                             
    #declare i = 1;
    #while(i<=2)
        sphere {
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size*3, 0, -top_box_size/2 + top_dot_size>
            top_dot_size
        }
        sphere {
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size*3, 0, top_box_size/2 - top_dot_size>
            top_dot_size
        }
        cylinder {
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size*3, 0, -top_box_size/2 + top_dot_size>
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size*3, 0, top_box_size/2 - top_dot_size>
            top_dot_size
        }
        #declare i = i+1;
    #end
    #declare i = 7;
    #while(i<=8)
        sphere {
            <-box_size + corner_width + top_dot_size*4*i + top_dot_size, 0, -top_box_size/2 + top_dot_size>
            top_dot_size
        }
        sphere {
            <-box_size + corner_width + top_dot_size*4*i + top_dot_size, 0, top_box_size/2 - top_dot_size>
            top_dot_size
        }
        cylinder {
            <-box_size + corner_width + top_dot_size*4*i + top_dot_size, 0, -top_box_size/2 + top_dot_size>
            <-box_size + corner_width + top_dot_size*4*i + top_dot_size, 0, top_box_size/2 - top_dot_size>
            top_dot_size
        }
        #declare i = i+1;
    #end
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}
       

  
#declare top_squares =
union {                             
    #declare i = 1;
    #while(i<=5)
        box{
            <-box_size + corner_width + top_box_size*(i-1) + top_box_space*i, -shallow_indent, -top_box_size/2>
            <-box_size + corner_width + top_box_size*i + top_box_space*i, shallow_indent, top_box_size/2>
        }
        #declare i = i+1;
    #end
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}

#declare top_povrays_logo =
union {                             
    object {
        Povray_Logo
        scale <top_box_size/2, top_box_size/2, shallow_indent*8>
        translate <-box_size + corner_width + + top_box_space, -shallow_indent, 0>
    }
    object {
        Povray_Logo
        scale <top_box_size/2, top_box_size/2, shallow_indent*8>
        translate <-box_size + corner_width + top_box_size*4 + top_box_space*5, -shallow_indent, 0>
    }
    rotate x*45
    translate <top_box_size/2, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}

#declare top_povrays =
union {                             
    #declare i = 1;
    #while(i<=5)
        object {
            Povray_Logo
            scale <top_box_size/2, top_box_size/2, shallow_indent*8>
            translate <-box_size + corner_width + top_box_size*(i-1) + top_box_space*i, -shallow_indent, 0>
        }
        #declare i = i+1;
    #end
    rotate x*45
    translate <top_box_size/2, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}

#declare top_circles =
union {                             
    #declare i = 1;
    #while(i<=5)
        sphere {
            <-box_size + corner_width + top_box_size*(i-1/2) + top_box_space*i, 0, 0>
            top_box_size/2
        }
        #declare i = i+1;
    #end
    scale <1, 2/top_box_size*shallow_indent, 1>
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}
    
#declare top_triangles =
union {                             
    #declare i = 1;
    #while(i<=5)
        prism {
            linear_spline
            -shallow_indent, shallow_indent, 4
            <-box_size + corner_width + top_box_size*(i-1) + top_box_space*i, top_box_size/2>
            <-box_size + corner_width + top_box_size*i + top_box_space*i, top_box_size/2>
            <-box_size + corner_width + top_box_size*(i-0.5) + top_box_space*i, -top_box_size/2>
            <-box_size + corner_width + top_box_size*(i-1) + top_box_space*i, top_box_size/2>
        }
        #declare i = i+1;
    #end
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}

#declare top_dots =
union {                             
    #declare i = 1;
    #while(i<=8)
        sphere {
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size, 0, -top_box_size/2 + top_dot_size>
            top_dot_size
        }
        sphere {
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size, 0, top_box_size/2 - top_dot_size>
            top_dot_size
        }
        #declare i = i+1;
    #end
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}

#declare bottom_small_stripe_size = (box_size*2 - 2*corner_width)/25;
#declare bottom_small_stripes =
union {                             
    #declare i = 1;
    #while(i<=12)
        sphere {
            <-box_size + corner_width + bottom_small_stripe_size*2*i - bottom_small_stripe_size/2, 0, -top_box_size/2 + bottom_small_stripe_size/2>
            bottom_small_stripe_size/2
        }
        sphere {
            <-box_size + corner_width + bottom_small_stripe_size*2*i - bottom_small_stripe_size/2, 0, top_box_size/2 - bottom_small_stripe_size/2>
            bottom_small_stripe_size/2
        }
        cylinder {
            <-box_size + corner_width + bottom_small_stripe_size*2*i - bottom_small_stripe_size/2, 0, -top_box_size/2 + bottom_small_stripe_size/2>
            <-box_size + corner_width + bottom_small_stripe_size*2*i - bottom_small_stripe_size/2, 0, top_box_size/2 - bottom_small_stripe_size/2>
            bottom_small_stripe_size/2
        }
        #declare i = i+1;
    #end
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}

#declare bottom_horizontal_stripes =
union {                             
    #declare i = 1;
    #while(i<=5)
        sphere {
            <-box_size + corner_width + top_box_size*(i-1) + top_box_space*i + bottom_small_stripe_size/2, 0, -top_box_size/2 + bottom_small_stripe_size/2>
            bottom_small_stripe_size/2
        }
        sphere {
            <-box_size + corner_width + top_box_size*i + top_box_space*i - bottom_small_stripe_size/2, 0, -top_box_size/2 + bottom_small_stripe_size/2>
            bottom_small_stripe_size/2
        }
        cylinder {
            <-box_size + corner_width + top_box_size*(i-1) + top_box_space*i + bottom_small_stripe_size/2, 0, -top_box_size/2 + bottom_small_stripe_size/2>
            <-box_size + corner_width + top_box_size*i + top_box_space*i - bottom_small_stripe_size/2, 0, -top_box_size/2 + bottom_small_stripe_size/2>
            bottom_small_stripe_size/2
        }
        sphere {
            <-box_size + corner_width + top_box_size*(i-1) + top_box_space*i + bottom_small_stripe_size/2, 0, top_box_size/2 - bottom_small_stripe_size/2>
            bottom_small_stripe_size/2
        }
        sphere {
            <-box_size + corner_width + top_box_size*i + top_box_space*i - bottom_small_stripe_size/2, 0, top_box_size/2 - bottom_small_stripe_size/2>
            bottom_small_stripe_size/2
        }
        cylinder {
            <-box_size + corner_width + top_box_size*(i-1) + top_box_space*i + bottom_small_stripe_size/2, 0, top_box_size/2 - bottom_small_stripe_size/2>
            <-box_size + corner_width + top_box_size*i + top_box_space*i - bottom_small_stripe_size/2, 0, top_box_size/2 - bottom_small_stripe_size/2>
            bottom_small_stripe_size/2
        }
        #declare i = i+1;
    #end
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}    

#declare bottom_large_stripes =
union {                             
    #declare i = 1;
    #while(i<=8)
        sphere {
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size, 0, -top_box_size/2 + top_dot_size>
            top_dot_size
        }
        sphere {
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size, 0, top_box_size/2 - top_dot_size>
            top_dot_size
        }
        cylinder {
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size, 0, -top_box_size/2 + top_dot_size>
            <-box_size + corner_width + top_dot_size*4*i - top_dot_size, 0, top_box_size/2 - top_dot_size>
            top_dot_size
        }
        #declare i = i+1;
    #end
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}

#declare bottom_dot_size = (box_size*2 - 2*corner_width)/30;
#declare bottom_dots =
union {                             
    #declare i = 1;
    #while(i<=13)
        #if (even(i))
            sphere {
                <-box_size + corner_width + bottom_dot_size*2*i + bottom_dot_size, 0, 0>
                top_dot_size
            }
        #else
            sphere {
                <-box_size + corner_width + bottom_dot_size*2*i + bottom_dot_size, 0, -top_box_size/2 + bottom_dot_size>
                top_dot_size
            }
            sphere {
                <-box_size + corner_width + bottom_dot_size*2*i + bottom_dot_size, 0, top_box_size/2 - bottom_dot_size>
                top_dot_size
            } 
        #end
        #declare i = i+1;
    #end
    rotate x*-45
    translate <0, 2*cap_height/3 + small_corner_width*sind(45), -box_size + small_corner_width*(1-sind(45)) + cap_height/3>
}
    
#declare bottom_border = corner_width/3;
#declare bottom_bump_size = (box_size - corner_width - bottom_border)/12;
#declare bottom_bumps =
difference {
    union {
        cylinder {
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width+deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            front_dot_size
        }   
        cylinder {
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width+deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            front_dot_size
        }   
        cylinder {
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width-deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width+deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
            front_dot_size
        }   
        cylinder {
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width-deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width+deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
            front_dot_size
        }   
        box {
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border>
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width+deeper_indent, -box_size + corner_width + bottom_border + front_dot_size+0.001>
        }
        box {
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width-deeper_indent, box_size - corner_width - bottom_border>
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width+deeper_indent, box_size - corner_width - bottom_border - front_dot_size-0.001>
        }
        box {
            <-box_size + corner_width + bottom_border, cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            <box_size - corner_width - bottom_border, cap_height+small_corner_width+deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
        }
    }
    union {
        #declare i = 1;
        #while(i<=6)
            #declare j = 1;
            #while(j<=6)
                sphere {
                    <-box_size + corner_width + bottom_border + bottom_bump_size*(4*i-2), cap_height+small_corner_width-bottom_bump_size, -box_size + corner_width + bottom_border + bottom_bump_size*(4*j-2)>
                    bottom_bump_size
                }
                cylinder {
                    <-box_size + corner_width + bottom_border + bottom_bump_size*(4*i-2), cap_height+small_corner_width-bottom_bump_size, -box_size + corner_width + bottom_border + bottom_bump_size*(4*j-2)>
                    <-box_size + corner_width + bottom_border + bottom_bump_size*(4*i-2), cap_height+small_corner_width-deeper_indent-0.001, -box_size + corner_width + bottom_border + bottom_bump_size*(4*j-2)>
                    bottom_bump_size
                }
                #declare j = j+1;
            #end
            #declare i = i+1;
        #end
    }
}

#declare top_ridge_size = (box_size - corner_width - bottom_border)/11;
#declare top_ridges =
difference {
    union {
        cylinder {
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width+deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            front_dot_size
        }   
        cylinder {
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width+deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            front_dot_size
        }   
        cylinder {
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width-deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width+deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
            front_dot_size
        }   
        cylinder {
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width-deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width+deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
            front_dot_size
        }   
        box {
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border>
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width+deeper_indent, -box_size + corner_width + bottom_border + front_dot_size+0.001>
        }
        box {
            <-box_size + corner_width + bottom_border + front_dot_size, cap_height+small_corner_width-deeper_indent, box_size - corner_width - bottom_border>
            <box_size - corner_width - bottom_border - front_dot_size, cap_height+small_corner_width+deeper_indent, box_size - corner_width - bottom_border - front_dot_size-0.001>
        }
        box {
            <-box_size + corner_width + bottom_border, cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border + front_dot_size>
            <box_size - corner_width - bottom_border, cap_height+small_corner_width+deeper_indent, box_size - corner_width - bottom_border - front_dot_size>
        }
    }
    union {
        #declare i = 1;
        #while(i<=7)
            sphere {
                <-box_size + corner_width + bottom_border + top_ridge_size*(3*i-1), cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border + top_ridge_size*2>
                top_ridge_size
            }
            sphere {
                <-box_size + corner_width + bottom_border + top_ridge_size*(3*i-1), cap_height+small_corner_width-deeper_indent, box_size - corner_width - bottom_border - top_ridge_size*2>
                top_ridge_size
            }
            cylinder {
                <-box_size + corner_width + bottom_border + top_ridge_size*(3*i-1), cap_height+small_corner_width-deeper_indent, -box_size + corner_width + bottom_border + top_ridge_size*2>
                <-box_size + corner_width + bottom_border + top_ridge_size*(3*i-1), cap_height+small_corner_width-deeper_indent, box_size - corner_width - bottom_border - top_ridge_size*2>
                top_ridge_size
            }
            #declare i = i+1;
        #end
    }
}

#declare finger_hole_size = (cylinder_size*2-wall_thickness*11)/4; 
#declare block_finger_holes =
difference {
    union {
        difference {
            cylinder {
                <0, box_size, 0>
                <0, glass_height-0.001, 0>
                cylinder_size-wall_thickness*4
            }
            cylinder {
                <0, box_size-shallow_indent, 0>
                <0, box_size+0.001, 0>
                cylinder_size-wall_thickness*5
            }
        }
        cylinder {
            <-wall_thickness/2-finger_hole_size, box_size,0>   
            <-wall_thickness/2-finger_hole_size, box_size*0.8,0>
            finger_hole_size+wall_thickness
        }   
        cylinder {
            <wall_thickness/2+finger_hole_size, box_size,0>   
            <wall_thickness/2+finger_hole_size, box_size*0.8,0>
            finger_hole_size+wall_thickness
        }
    }
    union {
        cylinder {
            <-wall_thickness/2-finger_hole_size, box_size+0.001,0>   
            <-wall_thickness/2-finger_hole_size, box_size*0.8,0>
            finger_hole_size
        }   
        cylinder {
            <wall_thickness/2+finger_hole_size, box_size+0.001,0>   
            <wall_thickness/2+finger_hole_size, box_size*0.8,0>
            finger_hole_size
        }
    }
}
 
#declare box_slit =
union {
    box {
        <-box_size + bottom_bump_size, -1, -bottom_bump_size>
        <box_size - bottom_bump_size, 1, bottom_bump_size>
    }
    cylinder {
        <-box_size + bottom_bump_size, -1, 0>
        <-box_size + bottom_bump_size, 1, 0>
        bottom_bump_size
    }
    cylinder {
        <box_size - bottom_bump_size, -1, 0>
        <box_size - bottom_bump_size, 1, 0>
        bottom_bump_size
    }
}
