#include "colors.inc"
#include "block_generic.inc"
#include "math.inc"
#include "glass.inc"

#declare simple_blocks=0;
#declare simple_textures=0;
#declare no_glass =0;
#declare use_photons=1;

/* reflection1
#macro block_plastic()
    #if (simple_textures)
        diffuse 0.6
    #else
        specular 1
        roughness 0.002
        //brilliance 0.3
        reflection {
            0, 0.12
            metallic 0.1
            exponent 1.0
        }
        diffuse 0.6
    #end
#end
*/
#macro block_plastic()
    #if (simple_textures)
        diffuse 0.6
    #else
        specular 1
        roughness 0.002
        //brilliance 0.3
        reflection {
            0, 0.12
            metallic 0.1
            exponent 1.1
        }
        diffuse 0.6
    #end
#end

#declare block_blue = color rgb <47/255, 159/255, 223/255>;
#declare block_sky_blue = color rgb <110/255, 215/255, 215/255>;
#declare block_lavendar = color rgb <146/255, 131/255, 190/255>; 
#declare block_lime = color rgb <130/255, 184/255, 90/255>;
#declare block_yellow = color rgb <255/255, 189/255, 60/255>;
#declare block_white = color rgb <1, 1, 1>;
#declare block_red = color rgb <176/255, 42/255, 43/255>;
#declare block_orange = color rgb <250/255, 90/255, 30/255>;


#declare block_size = box_size;

/* *************************
 * Metals are copied from metals.inc. They have been
 * modified to remove the ambient parameter.
 */

#declare F_MetalC  =
finish {
    brilliance 4
    diffuse 0.75
    metallic
    specular 0.80
    roughness 1/80
    reflection 0.5
}

#declare F_MetalE  =
finish {
    brilliance 6
    diffuse 0.8
    metallic
    specular 0.80
    roughness 1/120
    reflection 0.8
}

#declare P_Chrome1   = color rgb <0.20, 0.20, 0.20>;
#declare P_Chrome3   = color rgb <0.50, 0.50, 0.50>;

#declare T_Chrome_1C = texture { pigment { P_Chrome1 } finish { F_MetalC  } }
#declare T_Chrome_3C = texture { pigment { P_Chrome3 } finish { F_MetalC  } }
#declare T_Chrome_3E = texture { pigment { P_Chrome3 } finish { F_MetalE  } }


/* **************************************************
 *       STRIPEY CYLINDER BLOCK
 * **************************************************/
 
#declare stripey_cylinder_outer_size = cylinder_size + wall_thickness;
#declare stripey_cylinder_outer_thickness = wall_thickness/2;
#declare stripey_cylinder_inner_size = cylinder_size;
 
 
#macro stripey_cylinder_block_top()
difference {
    difference {
        object { block_cover }
        union {
            object { top_text_logo }
            object { top_front_dots }
            object {
                top_squares
                rotate y*-90
            }
            object {
                top_triangles
                rotate y*90
            }
            object {
                top_dots
                rotate y*180
            }
            split_union off
        }
    }
    cylinder {
        <0, box_size+1, 0>
        <0, box_size-1, 0>
        cylinder_size-wall_thickness*3
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    translate y*glass_height
    pigment { block_lime }
    finish { block_plastic() }
}
#end

#macro stripey_cylinder_block_bottom()
difference {
    union {
        object { block_cover }
        cylinder {
            <0, -glass_height-cap_height/2, 0>
            <0, -glass_height, 0>
            stripey_cylinder_inner_size
        }
    }
    union {
        object { bottom_small_stripes }
        object { bottom_bumps }
        object {
            bottom_horizontal_stripes
            rotate y*-90
        }
        object {
            bottom_large_stripes
            rotate y*90
        }
        object {
            bottom_dots
            rotate y*180
        }
        split_union off
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    rotate z*180
    translate y*-glass_height
    pigment { block_blue }
    finish { block_plastic() }
}
#end
    
        
              
#macro stripey_cylinder_outer()
    difference {
        cylinder {
            <0, -glass_height-cap_height/2, 0>
            <0, glass_height+cap_height/2, 0>
            stripey_cylinder_outer_size
        }
        cylinder {
            <0, -glass_height-cap_height/2 - .001, 0>
            <0, glass_height+cap_height/2 + .001, 0>
            stripey_cylinder_outer_size - stripey_cylinder_outer_thickness
        }  
        pigment {
            function {
                degrees(atan(z / x)) / 90 -
                ((glass_height - y) / (1.4*glass_height))
            }
            color_map {
                [0 / 12 color Clear]
                [1 / 12 color Clear]
                [1 / 12 color Black]
                [3 / 12 color Black]
                [3 / 12 color Clear]
                [5 / 12 color Clear]
                [5 / 12 color Black]
                [7 / 12 color Black]
                [7 / 12 color Clear]
                [9 / 12 color Clear]
                [9 / 12 color Black]
                [11 / 12 color Black]
                [11 / 12 color Clear]
                [12 / 12 color Clear]
            }
        }
        texture { block_glass_texture }
        interior { block_glass_interior }
        #if (use_photons != 0)
        #else
            photons {
                target use_photons
                refraction on
                reflection on
                collect off
            }
        #end
    }
#end

#macro stripey_cylinder_inner()
union {
    cylinder {
        <0, -glass_height, 0>
        <0, glass_height, 0>
        stripey_cylinder_inner_size
        pigment {
            function {
                degrees(atan(z / x)) / 90 +
                ((glass_height - y) / (1.4*glass_height))
            }
            color_map {
                [0 / 12 color White]
                [1 / 12 color White]
                [1 / 12 color Black]
                [3 / 12 color Black]
                [3 / 12 color White]
                [5 / 12 color White]
                [5 / 12 color Black]
                [7 / 12 color Black]
                [7 / 12 color White]
                [9 / 12 color White]
                [9 / 12 color Black]
                [11 / 12 color Black]
                [11 / 12 color White]
                [12 / 12 color White]
            }
        }
    }
    object {
        block_finger_holes
        pigment { block_yellow }
        finish { block_plastic() }
    }
    split_union off
}
#end

#macro stripey_cylinder_block(turn_angle)
    #if (simple_blocks)
        union {
            difference {
                object { block_cover }
                cylinder {
                    <0, box_size+1, 0>
                    <0, box_size-1, 0>
                    cylinder_size-wall_thickness*3
                }
                translate y*glass_height
                pigment { block_lime }
            }
            #if (! no_glass)
                object { glass_walls() }
                object { stripey_cylinder_outer() }
            #end
            object {
                block_cover
                rotate z*180
                translate y*-glass_height
                pigment { block_blue }
            }
            object {
                stripey_cylinder_inner()
                rotate y*turn_angle
            }
        }
    #else
        union {
            #if (! no_glass)
                object { glass_walls() }
                object { stripey_cylinder_outer() }
            #end
            object { stripey_cylinder_block_top() }
            object { stripey_cylinder_block_bottom() }
            object {
                stripey_cylinder_inner()
                rotate y*turn_angle
            }
        }
    #end
#end

/* **************************************************
 *       WAVY MIRROR BLOCK
 * **************************************************/

#declare mirror_corner_width = corner_width - 2*wall_thickness;
#declare mirror_size = (box_size-2*wall_thickness)/box_size;
#declare w_mir_width = (mirror_size - mirror_corner_width)*2;
#declare w_mir_r1 = (mirror_size - mirror_corner_width)/2.2;
//#declare w_mir_r2 = (mirror_size - mirror_corner_width)/1.5;
#declare w_mir_r2 = (2*pow(w_mir_r1,3)-6*pow(w_mir_r1,2)*w_mir_width+3*w_mir_r1*pow(w_mir_width,2)-2*sqrt(2)*sqrt(-2*pow(w_mir_r1,5)*w_mir_width+5*pow(w_mir_r1,4)*pow(w_mir_width,2)-4*pow(w_mir_r1,3)*pow(w_mir_width,3)+pow(w_mir_r1,2)*pow(w_mir_width,4)))/(2*pow(w_mir_r1,2));
#declare w_mir_r1=w_mir_r1;
#declare w_mir_r2=w_mir_r2;
#declare mirror =
object {
    round_corner_box
    scale <mirror_size, glass_height/box_size + 4*wall_thickness, mirror_size>
}


#macro wavy_mirror()
difference {
    object { mirror }
    // Concave - left side
    sphere {
        <0, 0, 0>
        box_size*1.2
        translate x*(-mirror_size - box_size*1.2 + mirror_size*0.15)
    }
    // Convex - right side
    difference {
        prism {
            -1,1,5,
            <-1,0>, <1,0>, <0.8,1>, <-0.8,1>, <-1,0>
        }
        sphere {
            <0,0,1.002>
            1
        }
        scale <mirror_size*0.75, glass_height+2*wall_thickness, 0.15>
        translate z*-mirror_size-0.001
        rotate y*-90
    }
    // weird - back
    intersection {
        difference {
            union {
                box {
                    <-w_mir_width/2, -box_size, box_size> 
                    <w_mir_width/2, box_size, mirror_size - w_mir_r1> 
                }
                cylinder {                 
                    <w_mir_width/2-sqrt(pow(w_mir_r2,2)-pow(w_mir_r2-w_mir_r1,2)), -box_size, mirror_size - 2*w_mir_r1 + w_mir_r2> 
                    <w_mir_width/2-sqrt(pow(w_mir_r2,2)-pow(w_mir_r2-w_mir_r1,2)), box_size, mirror_size - 2*w_mir_r1 + w_mir_r2> 
                    w_mir_r2
                }
            }
            cylinder {                 
                <-w_mir_width/2+w_mir_r1, -box_size, mirror_size - w_mir_r1> 
                <-w_mir_width/2+w_mir_r1, box_size, mirror_size - w_mir_r1> 
                w_mir_r1
            }
        }
        box {
            <-w_mir_width/2+0.001, -glass_height - wall_thickness, box_size>
            <w_mir_width/2-0.001, glass_height + wall_thickness, 0> 
        }
    }
    #if (simple_textures)
        pigment { White }
    #else
        texture {
            T_Chrome_3E
            normal { bumps 0.1 }
            normal { dents 0.05 }
            normal { ripples 0.15 }
        } 
        #if (use_photons = 0)
            photons {
                collect off
            }
        #else
            photons {
                target use_photons
                reflection on
                collect off
            }
        #end
    #end
}
#end
 
#macro wavy_mirror_block_top()
difference {
    object { block_cover }
    union {
        object { top_text_logo }
        object { top_front_dots }
        object { top_ridges }
        object {
            top_squares
            rotate y*-90
        }
        object {
            top_triangles
            rotate y*90
        }
        object {
            top_dots
            rotate y*180
        }
        split_union off
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    translate y*glass_height
    pigment { block_blue }
    finish { block_plastic() }
}
#end



#macro wavy_mirror_block_bottom()
difference {
    object { block_cover }
    union {
        object { bottom_small_stripes }
        object { bottom_bumps }
        object {
            bottom_horizontal_stripes
            rotate y*-90
        }
        object {
            bottom_large_stripes
            rotate y*90
        }
        object {
            bottom_dots
            rotate y*180
        }
        split_union off
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    rotate z*180
    translate y*-glass_height
    pigment { block_lime }
    finish { block_plastic() }
}
#end

#macro wavy_mirror_block()
    #if (simple_blocks)
        union {
            #if (! no_glass)
                object { glass_walls() }
            #end
            object {
                block_cover
                translate y*glass_height
                pigment { block_blue }
            }
            object {
                block_cover
                rotate z*180
                translate y*-glass_height
                pigment { block_lime }
            }
            object { wavy_mirror() }
        }
    #else
        union {
            #if (! no_glass)
                object { glass_walls() }
            #end
            object { wavy_mirror_block_top() }
            object { wavy_mirror_block_bottom() }
            object { wavy_mirror() }
        }
    #end
#end



/* **************************************************
 *       BALLOON BLOCK
 * **************************************************/


#macro balloon_block_top()
difference {
    object { block_cover }
    intersection {
        object {
            round_corner_box
            scale <0.99, 2, 0.99>
            translate y*-box_size
        }
        union {
            object {
                rough_block_cover
                scale <1-2*wall_thickness, 1.0, 1-2*wall_thickness>
                translate y*-0.001
            }
            object {
                rough_block_cover
                scale <1-2*wall_thickness, 1.0, 1-2*wall_thickness>
                translate y*0.001
            }
        }
    }
    union {
        object { top_text_logo }
        object { top_front_large_stripes }
        //object { top_ridges }
        object {
            top_triangles
            rotate y*-90
        }
        object {
            bottom_horizontal_stripes
            rotate y*90
        }
        object {
            top_circles
            rotate y*180
        }
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    translate y*glass_height
    texture {
        pigment { block_lavendar }
        finish { block_plastic() }
    }
}
#end

#macro balloon_block_top_window()
    intersection {
        difference {
            object {
                rough_block_cover
                scale <1-2*wall_thickness, 1.0, 1-2*wall_thickness>
            }
            object {
                rough_block_cover
                scale <1-4*wall_thickness, 1-wall_thickness, 1-4*wall_thickness>
                translate y*-0.001
            }
        }
        object {
            round_corner_box
            scale <0.99, 2, 0.99>
            translate y*-box_size
        }
        translate y*glass_height
        texture { block_glass_texture }
        interior { block_glass_interior }
        #if (use_photons = 0)
            photons {
                pass_through
                collect off
            }
        #else
            #if (use_photons = 1)
                photons {
                    target use_photons
                    refraction on
                    reflection on
                    collect off
                }
            #else
                photons {
                    target use_photons
                    refraction on
                    collect off
                }
            #end
        #end
    }
#end

#macro balloon_block_bottom()
union {
    difference {
        object { block_cover }
        union {
            object { bottom_small_stripes }
            object { bottom_bumps }
            object {
                bottom_horizontal_stripes
                rotate y*-90
            }
            object {
                top_triangles
                rotate y*90
            }
            object {
                bottom_dots
                rotate y*180
            }
        }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    }
    cylinder {
        <0, 0, 0>
        <0, cap_height/3, 0>
        0.3
    }
    rotate z*180
    translate y*-glass_height
    texture {
        pigment { block_lime }
        finish { block_plastic() }
    }
}
#end

#macro balloon_block_string()
cylinder {
    <0, 0, 0>
    <0, -1, 0>
    0.01
    texture { T_Chrome_3E }
    /*#if (use_photons = 0)
        photons {
            collect off
        }
    #else
        photons {
            target use_photons
            reflection on
            collect off
        }
    #end*/
}
#end

#declare balloon_size = 0.35;
#declare balloon_block_balloon =
union {
    intersection
    {
        sphere {
            <0, 0, 0>
            balloon_size
        }
        box {
            <-balloon_size-0.001, -0.001, -balloon_size-0.001>
            <balloon_size+0.001, balloon_size+0.001, balloon_size+0.001>
        }
    }
    intersection
    {
        sphere {
            <0, 0, 0>
            balloon_size
        }
        box {
            <-balloon_size-0.001, -balloon_size-0.001, -balloon_size-0.001>
            <balloon_size+0.001, 0.001, balloon_size+0.001>
        }
        scale <1, 1.5, 1>
    }
    cone {
        <0, -balloon_size*1.5-0.1, 0>, 0.12
        <0, -balloon_size*1.5+0.1, 0>, 0.06
    }
}

#macro balloon_block()
    #if (simple_blocks)
        union {
            #if (! no_glass)
                object { glass_walls() }
                object { balloon_block_top_window() }
            #end
            difference {
                object { block_cover }
                intersection {
                    object {
                        round_corner_box
                        scale <0.99, 2, 0.99>
                        translate y*-box_size
                    }
                    union {
                        object {
                            rough_block_cover
                            scale <1-2*wall_thickness, 1.0, 1-2*wall_thickness>
                            translate y*-0.001
                        }
                        object {
                            rough_block_cover
                            scale <1-2*wall_thickness, 1.0, 1-2*wall_thickness>
                            translate y*0.001
                        }
                    }
                }
                pigment { block_lavendar }
                translate y*glass_height
            }

            union {
                object { block_cover }
                cylinder {
                    <0, 0, 0>
                    <0, cap_height/3, 0>
                    0.3
                }
                rotate z*180
                translate y*-glass_height
                pigment { block_lime }
            }
            object {
                balloon_block_balloon
                texture { 
                    pigment { block_yellow }
                }
                rotate x*-20
                translate <0, 0.3, -0.4>
            }  
            object {
                balloon_block_balloon
                texture {
                    pigment { block_red }
                }
                rotate x*-20
                rotate y*135
                translate <-0.4, 0.3, 0.4>
            }
            object {
                balloon_block_balloon
                texture {
                    pigment { block_blue }
                }
                rotate x*-20
                rotate y*-135
                translate <0.4, 0.3, 0.4>  
            }

        }
    #else
        union {
            #if (! no_glass)
                object { glass_walls() }
                object { balloon_block_top_window() }
            #end
            object { balloon_block_top() }
            object { balloon_block_bottom() }
            union {
                object { balloon_block_string() }
                object {
                    balloon_block_balloon
                    texture { 
                        pigment { block_yellow }
                        finish { block_plastic() }
                    }
                }
                rotate x*-20
                translate <0, 0.3, -0.4>
            }  
            union {
                object { balloon_block_string() }
                object {
                    balloon_block_balloon
                    texture {
                        pigment { block_red }
                        finish { block_plastic() }
                    }
                }        
                rotate x*-20
                rotate y*135
                translate <-0.4, 0.3, 0.4>
            }
            union {
                object { balloon_block_string() }
                object {
                    balloon_block_balloon
                    texture {
                        pigment { block_blue }
                        finish { block_plastic() }
                    }
                }        
                rotate x*-20
                rotate y*-135
                translate <0.4, 0.3, 0.4>  
            }
        }
    #end
#end



/* **************************************************
 *       CIRCUS CYLINDER BLOCK
 * **************************************************/
 
#declare circus_cylinder_size = cylinder_size;  

#macro circus_cylinder_block_top()
difference {
    object { block_cover }
    union {
        object { top_text_logo }
        object { top_front_large_stripes }
        object { top_ridges }
        object {
            top_squares
            rotate y*-90
        }
        object {
            top_triangles
            rotate y*90
        }
        object {
            top_dots
            rotate y*180
        }
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    translate y*glass_height
    texture { 
        pigment { block_lavendar }
        finish { block_plastic() }
    }
}
#end



#macro circus_cylinder_block_bottom()
difference {
    difference {
        object { block_cover }
        union {
            object { bottom_dots }
            object {
                bottom_horizontal_stripes
                rotate y*90
            }
            object {
                bottom_large_stripes
                rotate y*-90
            }
            object {
                bottom_small_stripes
                rotate y*180
            }
        }
    }
    cylinder {
        <0, box_size+1, 0>
        <0, box_size-1, 0>
        cylinder_size-wall_thickness*3
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    rotate z*180
    translate y*-glass_height
    texture { 
        pigment { block_lime }
        finish { block_plastic() }
    }
}
#end
    
        
              
#macro circus_cylinder()
union {
    cylinder {
        <0, -glass_height-cap_height/2+0.15, 0>
        <0, glass_height+cap_height/2-0.05, 0>
        circus_cylinder_size
        texture {
            pigment { image_map { jpeg "circus.jpg" map_type 2 interpolate 2 } }
            scale <1, 2*glass_height+cap_height-0.12, 1>
            translate y*(-glass_height-cap_height/2+0.1)
        }
    }
    cylinder {
        <0, -glass_height-cap_height/2+0.15, 0>
        <0, -glass_height-cap_height/2, 0>
        circus_cylinder_size + 0.05
        texture { 
            pigment { block_yellow }
            finish { block_plastic() }
        }
    }
    cylinder {
        <0, glass_height+cap_height/2, 0>
        <0, glass_height+cap_height/2-0.05, 0>
        circus_cylinder_size + 0.05
        texture {  
            pigment { block_yellow }
            finish { block_plastic() }
        }
    }
    object {
        block_finger_holes
        texture { 
            pigment { block_yellow }
            finish { block_plastic() }
        }    
        rotate x*180
    }
}
#end



#macro circus_cylinder_block(turn_angle)
    #if (simple_blocks)
        union {
            #if (! no_glass)
                object { glass_walls() }
            #end
            difference {
                object { block_cover }
                cylinder {
                    <0, box_size+1, 0>
                    <0, box_size-1, 0>
                    cylinder_size-wall_thickness*3
                }
                rotate z*180
                translate y*-glass_height
                pigment { block_lime }
            }
            object {
                block_cover
                translate y*glass_height
                pigment { block_lavendar }
            }
            object {
                circus_cylinder()
                rotate y*turn_angle
            }
        }
    #else
        union {
            #if (! no_glass)
                object { glass_walls() }
            #end
            object { circus_cylinder_block_top() }
            object { circus_cylinder_block_bottom() }
            object {
                circus_cylinder()
                rotate y*turn_angle
            }
        }
    #end
#end


/* **************************************************
 *       BELL BLOCK
 * **************************************************/
                             
#declare f_gaussian = function(x) { exp(-8 * pow(x, 2)) };                            
#declare bell_num_points = 50;
#declare bell_starting_point = 5;
#declare bell_extent = 12;

#macro bell()
union {
    lathe {
        linear_spline // linear_spline | quadratic_spline | cubic_spline
        bell_num_points+2, // number of points
        <0, f_gaussian(bell_starting_point/20)>
        #local i = bell_starting_point;
        #while (i < bell_extent)
            <(i/20), f_gaussian(i/20)>
            #local i = i + (bell_extent-bell_starting_point)/bell_num_points;
        #end
        <(i/20)+0.02, f_gaussian(i/20)-0.03>    
        scale <1, 1.2, 1>
    }
    prism {
        linear_spline
        -1, 1, 8
        <0, 0>, <2.5, 1>, <0, 2>,
        <0, 1.8>, <-0.2, 1.8>, <-0.2, 0.2>,
        <0, 0.2>, <0,0>
        translate <0, 0, -1>
        scale <0.25, 0.02, 0.3>
        rotate <0, 0, 90>
        rotate <0, -45, 0>
        translate <0, 0.75, 0>
    }
    translate y*(-box_size*0.5)
    #if (simple_textures)
        pigment { White }
    #else
        texture { T_Chrome_3C }
        #if (use_photons != 1)
            photons {
                collect off
            }
        #else
            photons {
                target use_photons
                reflection on
                collect off
            }
        #end
    #end
}
#end
                      
#declare bell_holder_cylinder = 0.13;                        
#declare bell_holder_width = bell_holder_cylinder*5;
#declare bell_holder =
union { 
    difference {
        union {
            cylinder {
                <-bell_holder_width/2, -0.05, 0>
                <-bell_holder_width/2, cap_height/2, 0>
                bell_holder_cylinder
            } 
            cylinder {
                <bell_holder_width/2, -0.05, 0>
                <bell_holder_width/2, cap_height/2, 0>
                bell_holder_cylinder
            }    
        }
        union {
            cylinder {
                <-bell_holder_width/2, -1, 0>
                <-bell_holder_width/2, cap_height/2, 0>
                bell_holder_cylinder/1.8
            } 
            cylinder {
                <bell_holder_width/2, -1, 0>
                <bell_holder_width/2, cap_height/2, 0>
                bell_holder_cylinder/1.8
            }    
        }
    }        
    difference { 
        box {
            <-bell_holder_width/2 + bell_holder_cylinder + 0.08, -0.1, -0.15>
            <bell_holder_width/2 - bell_holder_cylinder - 0.08, cap_height/2, 0.15>
        } 
        box {
            <-bell_holder_width/2 + bell_holder_cylinder + 0.1, -1, -0.13>
            <bell_holder_width/2 - bell_holder_cylinder - 0.1, 1, 0.13>
        }
    }
    union {
        cylinder {
            <-bell_holder_width/2, -0.07, 0>
            <-bell_holder_width/2, cap_height/2, 0>
            bell_holder_cylinder/2
        } 
        cylinder {
            <bell_holder_width/2, -0.07, 0>
            <bell_holder_width/2, cap_height/2, 0>
            bell_holder_cylinder/2
        }
        intersection {
            torus {
                0.1,
                bell_holder_cylinder/2
                rotate x*90
            }
            box {
                <-1, 0, -1>
                <0, -1, 1>
            }    
            translate <-bell_holder_width/2 + 0.1, -0.07, 0>
        }
        intersection {
            torus {
                0.1,
                bell_holder_cylinder/2
                rotate x*90
            }
            box {
                <0, 0, -1>
                <1, -1, 1>
            }    
            translate <bell_holder_width/2 - 0.1, -0.07, 0>
        } 
        cylinder {
            <-bell_holder_width/2 + 0.1, -0.17, 0>
            <bell_holder_width/2 - 0.1, -0.17, 0>
            bell_holder_cylinder/2
        }

    }
}
                           
#macro bell_block_top()
union {
    difference {
        object { block_cover }
        union {
            object { top_text_logo }
            object { top_front_dots }
            object { top_ridges }
            object {
                top_squares
                rotate y*-90
            }
            object {
                top_triangles
                rotate y*90
            }
            object {
                top_dots
                rotate y*180
            }
        }
        bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    } 
    object {
        bell_holder 
        rotate y*-45
    }
    translate y*glass_height
    texture { 
        pigment { block_sky_blue }
        finish { block_plastic() }
    }
}
#end

#macro bell_block_bottom()
difference {
    object { block_cover }
    union {
        object { bottom_dots }
        object { bottom_bumps }
        object {
            bottom_large_stripes
            rotate y*-90
        }
        object {
            bottom_horizontal_stripes
            rotate y*90
        }
        object {
            bottom_small_stripes
            rotate y*180
        }
        object {
            box_slit
            scale <0.5, 1, 1>
            translate z*(-bottom_bump_size*8)
        }
        object {
            box_slit
            scale <0.3, 1, 1>
            translate z*(-bottom_bump_size*4)
        }
        object {
            box_slit
            scale <0.3, 1, 1>
        }
        object {
            box_slit
            scale <0.3, 1, 1>
            translate z*(bottom_bump_size*4)
        }
        object {
            box_slit
            scale <0.5, 1, 1>
            translate z*(bottom_bump_size*8)
        }
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    rotate z*180
    translate y*-glass_height
    texture {
        pigment { block_lavendar }
        finish { block_plastic() }
    }
}
#end

#macro bell_block()
    #if (simple_blocks)
        union {
            #if (! no_glass)
                object { glass_walls() }
            #end
            union {
                object { block_cover }
                object {
                    bell_holder 
                    rotate y*-45
                }
                translate y*glass_height
                pigment { block_sky_blue }
            }
            object {
                block_cover
                rotate z*180
                translate y*-glass_height
                pigment { block_lavendar }
            }
            object { bell() }
        }
    #else
        union {
            #if (! no_glass)
                object { glass_walls() }
            #end
            object { bell_block_top() }
            object { bell_block_bottom() }
            object { bell() }
        }
    #end
#end
                             


/* **************************************************
 *       SMILEY BLOCK
 * **************************************************/
                             
#macro smiley_block_top()
difference {
    object { block_cover }
    union {
        object { top_text_logo }
        object { top_front_dots }
        object { top_ridges }
        object {
            top_triangles
            rotate y*-90
        }
        object {
            bottom_horizontal_stripes
            rotate y*90
        }
        object {
            top_circles
            rotate y*180
        }
        object {
            round_corner_box
            scale <box_size-wall_thickness, cap_height/2, box_size-wall_thickness>
        }
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    translate y*glass_height
    texture { 
        pigment { block_blue }
        finish { block_plastic() }
    }
}
#end

#macro smiley_block_bottom()
difference {
    object { block_cover }
    union {
        object { top_circles }
        object { bottom_bumps }
        object {
            top_triangles
            rotate y*-90
        }
        object {
            bottom_horizontal_stripes
            rotate y*90
        }
        object {
            bottom_small_stripes
            rotate y*180
        }
        object {
            round_corner_box
            scale <box_size-wall_thickness, cap_height/2, box_size-wall_thickness>
        }
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    rotate z*180
    translate y*-glass_height
    texture {
        pigment { block_lime }
        finish { block_plastic() }
    }
}
#end

#declare smiley_side_cutout = 
intersection {
    union {
        cylinder {
            <0, glass_height*0.65, 0>
            <0, -glass_height*0.65, 0>
            glass_height/7
        }
        sphere {
            <0, glass_height*0.65, 0>
            glass_height/7
        }
        sphere {
            <0, -glass_height*0.65, 0>
            glass_height/7
        }
    }
    box {
        <-glass_height, -glass_height, -box_size*0.012>
        <glass_height, glass_height, box_size*0.012>
    }
}

#macro smiley_middle()
difference {
    difference {
        union {
            object {
                round_corner_box
                scale <1, glass_height/(box_size*2), 1>
                translate y*(-(glass_height/(box_size*2))-0.0015)
                texture {
                    pigment { block_orange }
                    finish { block_plastic() }
                }
            }
            object {
                round_corner_box
                scale <1, glass_height/(box_size*2), 1>
                translate y*((glass_height/(box_size*2))+0.0015)
            }
        }
        difference {
            box {
                <-glass_height, -glass_height*0.9, box_size*0.98>
                <glass_height, glass_height*0.9, box_size+1>
            }
            object {
                smiley_side_cutout
                translate <0, 0, box_size*0.991>
            }
            object {
                smiley_side_cutout
                translate <4*glass_height/7, 0, box_size*0.991>
            }
            object {
                smiley_side_cutout
                translate <-4*glass_height/7, 0, box_size*0.991>
            }
        }
        difference {
            box {
                <-glass_height, -glass_height*0.9, -box_size*0.98>
                <glass_height, glass_height*0.9, -box_size-1>
            }
            object {
                smiley_side_cutout
                translate <0, 0, -box_size*0.991>
            }
            object {
                smiley_side_cutout
                translate <4*glass_height/7, 0, -box_size*0.991>
            }
            object {
                smiley_side_cutout
                translate <-4*glass_height/7, 0, -box_size*0.991>
            }
        }
        scale <0.99, 1, 0.99>
    }
    union {
        cone {
            <0, 1.5, 0>, 0
            <0, -2, 0>, 0.7
            rotate <0, 0, 90>
        }
        cone {
            <0, 1.5, 0>, 0
            <0, -2, 0>, 0.7
            rotate <0, 0, -90>
        }
        sphere {
            <0, 0, 0>
            box_size * 0.95
        }                     
    }
    bounded_by { box { <-box_size, -glass_height, -box_size>, <box_size, glass_height, box_size> } }
    texture {
        pigment { block_orange }
        finish { block_plastic() }
    }
    rotate <0, 90, 0>
}
#end

#macro smiley_face()
intersection {
    union {
        intersection {
            sphere {
                <0, 0, 0>
                box_size * 0.93
            }
            box {
                <-box_size, -box_size, -box_size>
                <0, box_size, box_size>
            }
            #if (simple_textures)
                pigment { block_yellow }
            #else
                pigment {
                    image_map { sys "Yellow-smile.bmp" interpolate 2 }
                    rotate y*90
                    scale <2,2,2>
                    translate <-1, -1, -1>
                }
            #end
        }
        intersection {
            sphere {
                <0, 0, 0>
                box_size * 0.93
            }                     
            box {
                <0, -box_size, -box_size>
                <box_size, box_size, box_size>
            }
        }
        #if (simple_textures)
            pigment { block_white }
        #else
            pigment {
                image_map { sys "White-smile.bmp" interpolate 2 }
                rotate y*90
                scale <2,2,2>
                translate <-1, -1, -1>
            }
        #end
    }
    box {
        <-box_size, -glass_height, -box_size>
        <box_size, glass_height, box_size>
    }
    pigment { block_white }
    finish { block_plastic() }
    scale <0.99, 0.99, 0.99>
}
#end

#macro smiley_block(turn_angle)
    #if (simple_blocks)
        union {
            object { smiley_middle() }
            object {
                smiley_face()
                rotate y*turn_angle
            }
            object {
                block_cover
                translate y*glass_height
                pigment { block_blue }
            }
            object {
                block_cover
                rotate z*180
                translate y*-glass_height
                pigment { block_lime }
            }
        }
    #else
        union {
            object { smiley_middle() }
            object {
                smiley_face()
                rotate y*turn_angle
            }
            object { smiley_block_top() }
            object { smiley_block_bottom() }
        }
    #end
#end




/* **************************************************
 *       TWO GEAR BLOCK
 * **************************************************/
 
 
#macro two_gear_block_top()
difference {
    object { block_cover }
    union {
        object { top_text_logo }
        object { top_front_dots }
        object { top_ridges }
        object {
            bottom_horizontal_stripes
            rotate y*90
        }
        object {
            top_triangles
            rotate y*-90
        }
        object {
            top_circles
            rotate y*180
        }
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    translate y*glass_height
    texture {
        pigment { block_orange }
        finish { block_plastic() }
    }
}
#end



#macro two_gear_block_bottom()
difference {
    object { block_cover }
    union {
        object { bottom_small_stripes }
        object { bottom_bumps }
        object {
            bottom_horizontal_stripes
            rotate y*-90
        }
        object {
            bottom_large_stripes
            rotate y*90
        }
        object {
            bottom_dots
            rotate y*180
        }
    }
    difference {
        cylinder {
            <0, -box_size, 0>, <0, box_size, 0>
            box_size
        }
        cylinder {
            <0,-box_size*2, 0>, <0, box_size*2, 0>
            box_size
            translate <-box_size*2+0.05, 0, 0>
        }
        translate <box_size*2-0.1*box_size, 0, 0>
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    rotate z*180
    translate y*-glass_height
    texture {
        pigment { block_lavendar }
        finish { block_plastic() }
    }
}
#end

#macro two_gear_block_glass_walls()
    merge {
        difference {
            object { glass_walls() }
            sphere { <-box_size*2+0.05+wall_thickness, -glass_height, 0>, box_size }
        }
        intersection {
            difference {
                sphere {
                    <-box_size*2+0.05+wall_thickness, -glass_height, 0>,
                    box_size
                }
                sphere {
                    <-box_size*2+0.05+wall_thickness, -glass_height, 0>,
                    box_size - wall_thickness
                }
            }
            object {
                clear_round_corner_box
                scale <1, glass_height*0.9999/box_size, 1>
            }
            texture { block_glass_texture }
            interior { block_glass_interior }
        }
        pigment { Clear }
        #if (use_photons = 0)
            photons {
                pass_through
                collect off
            }
        #else
            #if (use_photons = 1)
                photons {
                    target use_photons
                    refraction on
                    reflection on
                    collect off
                }
            #else
                photons {
                    target use_photons
                    refraction on
                    collect off
                }
            #end
        #end
    }
#end



#macro gear(gear_radius, num_teeth, tooth_height)
    isosurface {
        function {
            x*x + z*z -
            pow( (gear_radius +
                  sin( 2*pi*(atan2d(z,x)/360)*num_teeth) *
                  gear_radius * tooth_height
              ), 2)
            }
        accuracy 0.001
        contained_by {
            box {
                <-gear_radius*(1+tooth_height), -0.5,
                 -gear_radius*(1+tooth_height)>,
                <gear_radius*(1+tooth_height), 0.5,
                 gear_radius*(1+tooth_height)>
            }
        }
    }
#end
        
#macro two_gear_block_cylinder1(turn_angle)
union {
    union {
        object {
            gear(box_size*3/5, 30, 1/30)
            scale <1, wall_thickness, 1>
            translate y*(-glass_height-wall_thickness/2-0.01)
        }
        object {
            gear((box_size-wall_thickness-0.01)*0.5, 12, 1/12)
            scale <1, wall_thickness, 1>
            translate y*(-glass_height-3*wall_thickness/2-0.01)
        }
        cylinder {
            <0, -glass_height-cap_height/2, 0>,
            <0, glass_height+cap_height/2, 0>
            box_size*0.2
        }
        pigment { block_yellow }
        finish { block_plastic() }
    }
    cylinder {
        <0, -0.5, 0>
        <0, 0.5, 0>
        1
        pigment { 
            function {
                int((atan2d(z, x) + 180) / 360 * 14 + int((y+0.5)*6)) / 19
            }
            color_map {
                [0 color Yellow ]
                [1/19 color White ]
                [2/19 color Orange ]
                [3/19 color White ]
                [4/19 color Red ]
                [5/19 color White ]
                [6/19 color Magenta ]
                [7/19 color White ]
                [8/19 color Blue ]
                [9/19 color White ]
                [10/19 color Cyan ]
                [11/19 color White ]
                [12/19 color Green ]
                [13/19 color White ]
                [14/19 color Yellow ]
                [15/19 color White ]
                [16/19 color Orange ]
                [17/19 color White ]
                [18/19 color Red ]
            }
        }
        scale <(box_size - wall_thickness*7)/2,
        2*glass_height + cap_height/2 - 0.1, 
        (box_size - wall_thickness*7)/2>
        translate y*(cap_height/4)
    }
    rotate y*turn_angle
    translate x*-box_size*2/5
}
#end

#macro two_gear_block_cylinder2(turn_angle)
union {
    union {
        object {
            gear((box_size-wall_thickness-0.01)*0.5, 12, 1/12)
            scale <1, wall_thickness, 1>
            translate y*(-glass_height-cap_height/2+wall_thickness/2+0.01)
        }
        cylinder {
            <0, -glass_height-cap_height/2, 0>,
            <0, glass_height+cap_height/2, 0>
            box_size*0.2
        }
        pigment { block_yellow }
        finish { block_plastic() }
    }
    cylinder {
        <0, -0.5, 0>
        <0, 0.5, 0>
        1
        pigment { 
            function {
                int(
                    (atan2d(z, x) + 180) / 360 * 14 +
                    6-int((y+0.5)*6)
                ) / 19
            }
            color_map {
                [0 color Green ]
                [1/19 color White ]
                [2/19 color Cyan ]
                [3/19 color White ]
                [4/19 color Blue ] [5/19 color White ] [6/19 color Magenta ] [7/19 color White ] [8/19 color Red ] [9/19 color White ] [10/19 color Orange ] [11/19 color White ] [12/19 color Yellow ] [13/19 color White ] [14/19 color Green ] [15/19 color White ] [16/19 color Cyan ] [17/19 color White ]
                [18/19 color Blue ]
            }
        }
        scale <(box_size - wall_thickness*7)/2,
        2*glass_height + cap_height/2 - 0.1, 
        (box_size - wall_thickness*7)/2>
        translate y*(cap_height/4)
    }
    rotate y*turn_angle
    translate x*box_size*2/5
}
#end

#macro two_gear_block(turn_angle)
    #if (simple_blocks)
        union {
            #if (! no_glass)
                object { two_gear_block_glass_walls() }
            #end
            object {
                block_cover
                translate y*glass_height
                pigment { block_orange }
            }
            object {
                block_cover
                rotate z*180
                translate y*-glass_height
                pigment { block_lavendar }
            }
            object { two_gear_block_cylinder1(turn_angle) }
            object { two_gear_block_cylinder2(turn_angle) }
        }
    #else
        union {
            #if (! no_glass)
                object { two_gear_block_glass_walls() }
            #end
            object { two_gear_block_top() }
            object { two_gear_block_bottom() }
            object { two_gear_block_cylinder1(turn_angle) }
            object { two_gear_block_cylinder2(turn_angle) }
        }
    #end
#end


/* **************************************************
 *       TRIANGULAR PRISM BLOCK
 * **************************************************/
 
#macro triangular_prism_block_top()
difference {
    object { block_cover }
    union {
        object { top_text_logo }
        object { top_front_dots }
        object { top_ridges }
        object {
            top_triangles
            rotate y*90
        }
        object {
            top_squares
            rotate y*-90
        }
        object {
            top_dots
            rotate y*180
        }
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    translate y*glass_height
    texture {
        pigment { block_orange }
        finish { block_plastic() }
    }
}
#end



#macro triangular_prism_block_bottom()
difference {
    object { block_cover }
    union {
        object { bottom_small_stripes }
        object { bottom_bumps }
        object {
            bottom_horizontal_stripes
            rotate y*90
        }
        object {
            top_triangles
            rotate y*-90
        }
        object {
            top_circles
            rotate y*180
        }
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    rotate z*180
    translate y*-glass_height
    texture {
        pigment { block_yellow }
        finish { block_plastic() }
    }
}
#end


#declare tpb_sphere = function { x*x + y*y + z*z - 1 }
#declare tpb_box = function { max((y*y-1),(x*x-1),(z*z-1)) }
#declare sphere_size = box_size*7;

#declare tbp_lens =
function {
    tpb_sphere(
        x/sphere_size,
        y/sphere_size,
        (z-sphere_size+box_size)/sphere_size
    )
}

#declare tbp_front_indent =
function {
    min (
        min(
            min(
                min(
                    tpb_box(
                        x/(box_size-corner_width+0.001),
                        y/(glass_height-0.001),
                        (z+box_size-wall_thickness*3/2)/wall_thickness
                    ),
                    tpb_box(
                        (x+box_size-corner_width-wall_thickness+0.001)/(2*wall_thickness),
                        y/(glass_height-0.001),
                        (z+box_size-wall_thickness)/(2*wall_thickness)
                    )
                ),
                tpb_box(
                    (x-box_size+corner_width+wall_thickness-0.001)/(2*wall_thickness),
                    y/(glass_height-0.001),
                    (z+box_size-wall_thickness)/(2*wall_thickness)
                )
            ),
            tpb_box(
                x/(box_size-corner_width+0.001),
                (y+glass_height-wall_thickness-0.001)/(2*wall_thickness),
                (z+box_size-wall_thickness)/(2*wall_thickness)
            )
        ),
        tpb_box(
            x/(box_size-corner_width+0.001),
            (y-glass_height+wall_thickness+0.001)/(2*wall_thickness),
            (z+box_size-wall_thickness)/(2*wall_thickness)
        )
    )
}


#declare front_lens =
isosurface {
    function { min(tbp_lens(x,y,z), tbp_front_indent(x,y,z)) }
    accuracy 0.001
    max_gradient 50
    contained_by { 
        box {
            <-box_size+corner_width-0.001, -glass_height, -box_size>
            <box_size-corner_width+0.001, glass_height, -box_size+wall_thickness*2>
        }
    }
}

#declare side_prism =
prism {
    -1,1,12,
    <-box_size+corner_width,-box_size>,
    <-box_size+corner_width,-box_size+wall_thickness>,
    <(-box_size+corner_width)*3/4,-box_size+(box_size-corner_width)/8+wall_thickness>,
    <(-box_size+corner_width)/2,-box_size+wall_thickness>,
    <(-box_size+corner_width)/4,-box_size+(box_size-corner_width)/8+wall_thickness>,
    <0,-box_size+wall_thickness>,
    <(box_size-corner_width)/4,-box_size+(box_size-corner_width)/8+wall_thickness>,
    <(box_size-corner_width)/2,-box_size+wall_thickness>,
    <(box_size-corner_width)*3/4,-box_size+(box_size-corner_width)/8+wall_thickness>,
    <box_size-corner_width,-box_size+wall_thickness>,
    <box_size-corner_width,-box_size>,
    <-box_size+corner_width,-box_size>
    scale <1,glass_height*0.9999,1>
    rotate y*-90
}

#declare side_cylinders =
difference {
    merge {
        cylinder {
            <0,-1,0>,
            <0,1,0>,
            (box_size-corner_width)/4+0.001
            scale <0.5,glass_height*0.9999,1>
            translate <(-box_size+corner_width)*3/4,0,-box_size+wall_thickness>
        }
        cylinder {
            <0,-1,0>,
            <0,1,0>,
            (box_size-corner_width)/4+0.001
            scale <0.5,glass_height*0.9999,1>
            translate <(-box_size+corner_width)/4,0,-box_size+wall_thickness>
        }
        cylinder {
            <0,-1,0>,
            <0,1,0>,
            (box_size-corner_width)/4+0.001
            scale <0.5,glass_height*0.9999,1>
            translate <(box_size-corner_width)*3/4,0,-box_size+wall_thickness>
        }
        cylinder {
            <0,-1,0>,
            <0,1,0>,
            (box_size-corner_width)/4+0.001
            scale <0.5,glass_height*0.9999,1>
            translate <(box_size-corner_width)/4,0,-box_size+wall_thickness>
        }
    }
    box {
        <-box_size, -glass_height, -box_size>
        <box_size, glass_height, -2*box_size>
    }
    rotate y*90
}

#macro triangular_prism_block_glass_walls()
    merge {
        difference {
            object { glass_walls() }
            box {
                <-box_size-0.001, -1, -box_size+corner_width-0.001>
                <box_size+0.001, 1, box_size-corner_width+0.001>
            }
            box {
                <-box_size+corner_width-0.001, -1, -box_size-0.001>
                <box_size-corner_width+0.001, 1, 0>
            }
        }
        object { front_lens }
        object { side_prism }
        object { side_cylinders }
        texture { block_glass_texture }
        interior { block_glass_interior }
        #if (use_photons = 0)
            photons {
                pass_through
                collect off
            }
        #else
            #if (use_photons = 1)
                photons {
                    target use_photons
                    refraction on
                    reflection on
                    collect off
                }
            #else
                photons {
                    target use_photons
                    refraction on
                    collect off
                }
            #end
        #end
    }
#end

#declare triangular_prism_size = box_size/2;
#declare triangular_prism_block_prism =
union {
    box {
        <-1, -1, -0.0001>
        <0, 1, 0.0001>
        pigment { checker White, Blue scale 0.25 translate -1 }
        rotate y*60
        translate x*0.5
    }
    box {
        <0, -1, -0.0001>
        <1, 1, 0.0001>
        pigment {
            image_map { sys "Polka-dots.bmp" }
            scale <1, 2, 1>
            translate y*1
        }
        rotate y*-60
        translate x*-0.5
    }
    box {
        <-0.5, -1, -0.0001>
        <0.5, 1, 0.0001>
        pigment {
            function { int((y+1)*8.5)/17 }
            pigment_map {
                [ 0 Red ]
                [ 1/17 Red ]
                [ 2/17 Yellow ]
                [ 3/17 Red ]
                [ 4/17 Red ]
                [ 5/17 Red ]
                [ 6/17 Yellow ]
                [ 7/17 Red ]
                [ 8/17 Red ]
                [ 9/17 Red ]
                [ 10/17 Yellow ]
                [ 11/17 Red ]
                [ 12/17 Red ]
                [ 13/17 Red ]
                [ 14/17 Yellow ]
                [ 15/17 Red ]
                [ 16/17 Red ]
            }
        }
    }
    translate z*-(0.5/tand(60))
    scale <1, glass_height+cap_height/2, 1>
}

#macro triangular_prism_block(turn_angle)
    #if (simple_blocks)
        union {
            #if (! no_glass)
                object { triangular_prism_block_glass_walls() }
            #end
            object {
                block_cover
                translate y*glass_height
                pigment { block_orange }
            }
            object {
                block_cover
                rotate z*180
                translate y*-glass_height
                pigment { block_yellow }
            }
            object {
                triangular_prism_block_prism
                rotate y*turn_angle
            }
        }
    #else
        union {
            #if (! no_glass)
                object { triangular_prism_block_glass_walls() }
            #end
            object { triangular_prism_block_top() }
            object { triangular_prism_block_bottom() }
            object {
                triangular_prism_block_prism
                rotate y*turn_angle
            }
        }
    #end
#end



/* **************************************************
 *       POVRAY BLOCK
 * **************************************************/

#macro povray_block_top()
difference {
    object { block_cover }
    object { top_ridges }
    object { top_POV_logo }
    object { top_povrays_logo }
    object {
        top_povrays
        rotate y*90
    }
    object {
        top_povrays
        rotate y*-90
    }
    object {
        top_povrays
        rotate y*180
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    translate y*glass_height
    pigment { block_red }
    finish { block_plastic() }
}
#end

#macro povray_block_bottom()
difference {
    object { block_cover }
    object { bottom_bumps }
    object { top_povrays }
    object {
        top_povrays
        rotate y*90
    }
    object {
        top_povrays
        rotate y*-90
    }
    object {
        top_povrays
        rotate y*180
    }
    bounded_by { box { <-box_size, 0, -box_size>, <box_size, cap_height + small_corner_width, box_size> } }
    rotate z*180
    translate y*-glass_height
    pigment { block_yellow }
    finish { block_plastic() }
}
#end

#macro povray_block_crystal()
    object {
        Povray_Logo
        scale <0.8, 0.7, 1>
        texture {
            block_glass_texture
            pigment { color rgbt <0.3,0.2,1,0.5> }
        }
        interior { block_glass_interior }
        /*#if (use_photons = 0)
            photons {
                pass_through
                collect off
            }
        #else
            photons {
                target use_photons
                refraction on
                reflection on
                collect off
            }
        #end*/
    }
#end

#macro povray_block(turn_angle)
    #if (simple_blocks)
        union {
            #if (! no_glass)
                object { glass_walls() }
            #end
            object { povray_block_top() }
            object { povray_block_bottom() }
            object {
                povray_block_crystal()
                rotate y*turn_angle
            }
        }
    #else
        union {
            #if (! no_glass)
                object { glass_walls() }
            #end
            object { povray_block_top() }
            object { povray_block_bottom() }
            object {
                povray_block_crystal()
                rotate y*turn_angle
            }
        }
    #end
#end

